<?php

use Shopware\Components\CSRFWhitelistAware;
use Acl\GreenField\Exceptions\GreenFieldResponseException;
use Acl\GreenField\Exceptions\GreenFieldSDKException;

class Shopware_Controllers_Backend_PostLabels extends Shopware_Controllers_Backend_ExtJs implements CSRFWhitelistAware
{
    protected $nameMapping = array(
        'SHIPPING_LABEL' => 'Versandlabel (PDF)',
        'RETURN_LABEL' => 'Retourenlabel (PDF)',
        'BOTH' => 'Retourenlabel & Versandlabel (PDF)',
        'UNDEFINED_NAME' => 'Undefined'
    );

    public function loadOrderLabelsZipAction()
    {
        $gf = $this->container->get("post_label_center.greenfield_service");
        $orderNumbers = $this->Request()->getParam('orderNumbers');
        $overriddenConfig = false;
        $payLoadData = [];

        foreach ($orderNumbers as $orderNumber) {
            $repository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
            $order = $repository->findOneBy(["number" => $orderNumber]);
            $dispatch = $order->getDispatch();
            $dispatchId = $dispatch->getId();
            $overriddenConfig = false;
            $shopId = $order->getShop()->getId();

            try {
                $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\ShippingSet::class);
                $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);

                if ($shippingSet) {
                    $overriddenConfig = $gf->getOverriddenConfig($shippingSet);
                }
            } catch (\Exception $e) {
                // eventual implement error handling
                return $e;
            }
            if ($overriddenConfig) {
                $gf->overrideAuth($overriddenConfig);
            }

            $currentOrder["data"][] = ["orderId" => $orderNumber, "shopId" => $shopId];
            $currentOrder["meta"] = $overriddenConfig;
            $payLoadData[] = $currentOrder;
        }

        try {
            // get all documents and filter them later
            $pdfDocumentsData = $gf->listMultiplePdfs(array_values($payLoadData));
        } catch (GreenFieldResponseException $e) {
            $response = json_decode($e->getResponse()->getBody());
            $errors = array(
                'error' => array(
                    'errorMessage' => $response->error->message,
                    'errorMessageExtended' => $response->error->extendedMessage
                )
            );
            $this->View()->assign([
                "success" => false,
                "data" => $errors,
                "total" => count($errors)
            ]);
            return;

        } catch (Exception $e) {
            $errors = array(
                'error' => array(
                    'errorMessage' => $e->getMessage(),
                    'errorMessageExtended' => $e->getMessage()
                )
            );
            $this->View()->assign([
                "success" => false,
                "data" => $errors,
                "total" => count($errors)
            ]);
            return;
        }
        $zip = new \ZipArchive();
        $zipFilename = "./POST_LABELS.zip";
        if ($zip->open($zipFilename, \ZipArchive::CREATE) !== TRUE) {
            exit("cannot open <$zipFilename>\n");
        }
        $pdfDocuments = $pdfDocumentsData->data[0]->multipleShipmentDocuments;
        foreach ($pdfDocuments as $pdfDocumentA) {
            foreach ((array)$pdfDocumentA->shipmentDocumentMixins as $orderId => $pdfDocumentsOrder) {
                foreach ($pdfDocumentsOrder as $count => $pdfDocument) {
                    if ($pdfDocument) {
                        if ($pdfDocument->fileContent) {
                            $fileName = $orderId . "-" . $count . "-" . $pdfDocument->documentType . '-' . time() . '.pdf';
                            $pdfContent = base64_decode($pdfDocument->fileContent);
                            $zip->addFromString($fileName, $pdfContent);
                        }

                        if ($pdfDocument->shippingContent) {
                            $fileName = $orderId . "-" . "ZollDokumente-" . $pdfDocument->documentType . '-' . time() . '.pdf';
                            $pdfContent = base64_decode($pdfDocument->shippingContent);
                            $zip->addFromString($fileName, $pdfContent);
                        }
                    }
                }
            }
        }
        $zip->close();
        $this->Front()->Plugins()->ViewRenderer()->setNoRender();
        $this->Front()->Plugins()->Json()->setRenderer(false);
        $response = $this->Response();
        $response->setHeader('Content-Type', 'application/zip');
        $response->setHeader('Content-Description', 'File Transfer');
        $response->setHeader('Content-Disposition', 'attachment; filename="' . $zipFilename . '"');
        $response->setHeader('Expires', '0');
        $response->setHeader('Cache-Control', 'must-revalidate');
        $response->setHeader('Pragma', 'public');
        $response->setHeader('Content-Transfer-Encoding', 'binary');
        $response->setHeader('Content-Length', filesize($zipFilename));

        $response->sendHeaders();
        readfile($zipFilename);
        //ignore_user_abort(true);
        //register_shutdown_function('unlink', $zipFilename);
        unlink($zipFilename);

    }

    public function loadOrderLabelsAction()
    {
        // @todo: uncomment and finalize once the API is prepared
        $gf = $this->container->get("post_label_center.greenfield_service");
        $orderNumber = intval($this->Request()->getParam('orderNumber'));
        $orderId = intval($this->Request()->getParam('orderId'));
        $order = Shopware()->Models()->find('Shopware\Models\Order\Order', $orderId);
        $dispatch = $order->getDispatch();
        $dispatchId = $dispatch->getId();
        $overriddenConfig = false;
        $shopId = $order->getShop()->getId();

        // used shipping set
        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\ShippingSet::class);
            $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);

            if ($shippingSet) {
                $overriddenConfig = $gf->getOverriddenConfig($shippingSet);
            }
        } catch (\Exception $e) {
            // eventual implement error handling
            return $e;
        }

        if ($overriddenConfig) {
            $gf->overrideAuth($overriddenConfig);
        }

        try {
            // get all documents and filter them later
            $pdfDocuments = $gf->listPdfs($orderNumber, $shopId);
        } catch (GreenFieldResponseException $e) {
            $response = json_decode($e->getResponse()->getBody());
            $errors = array(
                'error' => array(
                    'errorMessage' => $response->error->message,
                    'errorMessageExtended' => $response->error->extendedMessage
                )
            );
            $this->View()->assign([
                "success" => false,
                "data" => $errors,
                "total" => count($errors)
            ]);
            return;

        } catch (Exception $e) {
            $errors = array(
                'error' => array(
                    'errorMessage' => $e->getMessage(),
                    'errorMessageExtended' => $e->getMessage()
                )
            );
            $this->View()->assign([
                "success" => false,
                "data" => $errors,
                "total" => count($errors)
            ]);
            return;
        }

        // mark documents with "downloaded" flag
        // get order attributes
        $orderAttributes = $gf->getOrderAttributes($orderId);
        $labelsDownloadsList = isset($orderAttributes['post_labels_downloads']) ? json_decode($orderAttributes['post_labels_downloads']) : array();
        $documentList = array();
        foreach ($pdfDocuments->data as $pdfDocument) {
            if ($pdfDocument->canceled) {
                continue;
            }
            $date = date('Y-m-d H:i:s', substr($pdfDocument->updatedOn, 0, 10));
            foreach ($labelsDownloadsList as $labelDownload) {
                if ($labelDownload->documentid == $pdfDocument->id) {
                    $doc = array();
                    $doc['id'] = $pdfDocument->id;
                    $doc['documentId'] = $pdfDocument->id;
                    $doc['date'] = $date;
                    //$doc['name'] = $pdfDocument->documentType;
                    $type = $labelDownload->label_data->type ? $labelDownload->label_data->type : 'undefined_name';
                    $doc['name'] = $this->nameMapping[$type];
                    $doc['orderId'] = $orderId;
                    $doc['orderNumber'] = $orderNumber;
                    $doc['isShipping'] = false;
                    $labelDownload->label_data->downloaded ? $doc['downloaded'] = true : $doc['downloaded'] = false;
                    $documentList[] = $doc;
                    if ($pdfDocument->shippingContent) {
                        $doc2 = array();
                        $doc2['id'] = $pdfDocument->id . "9";
                        $doc2['documentId'] = $doc["documentId"];
                        $doc2['date'] = $date;
                        //$doc['name'] = $pdfDocument->documentType;
                        $type = $labelDownload->label_data->type ? $labelDownload->label_data->type : 'undefined_name';
                        $doc2['name'] = $this->nameMapping[$type];
                        $doc2['orderId'] = $orderId;
                        $doc2['orderNumber'] = $orderNumber;
                        $doc2['isShipping'] = true;

                        $doc2["name"] = "DOKUMENTE-" . $this->nameMapping[$type];

                        $documentList[] = $doc2;

                    }
                    break;
                }
            }
        }
        $this->View()->assign([
            "success" => true,
            "data" => $documentList,
            "total" => count($documentList)
        ]);
    }

    public function openPdfAction()
    {
        $gf = $this->container->get("post_label_center.greenfield_service");
        $documentId = $this->Request()->getParam('documentid');
        $isShipping = $this->Request()->getParam('isShipping');
        $orderNumber = intval($this->Request()->getParam('orderNumber'));
        $shopId = $this->Request()->getParam('shopId');
        $orderId = $this->Request()->getParam('orderid');
        $order = Shopware()->Models()->find('Shopware\Models\Order\Order', $orderId);
        $dispatch = $order->getDispatch();
        $dispatchId = $dispatch->getId();
        $overriddenConfig = false;

        if (preg_match('/\D/', $documentId)) {
            die('Invalid parameter');
        }


        // used shipping set
        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\ShippingSet::class);
            $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);

            if ($shippingSet) {
                $overriddenConfig = $gf->getOverriddenConfig($shippingSet);
            }
        } catch (\Exception $e) {
            // eventual implement error handling
            return $e;
        }

        if ($overriddenConfig) {
            $gf->overrideAuth($overriddenConfig);
        }

        try {
            $pdfDocument = $gf->loadPdf($documentId, $orderNumber, $shopId);

        } catch (Exception $e) {
            echo $e->getMessage();
            die;
        }
        if ($isShipping !== 'false') {
            $pdfContent = base64_decode($pdfDocument->shippingContent);
            $fileName = "DOKUMENTE" . '-' . date("d-m-Y", $pdfDocument->updatedOn / 1000) . '.pdf';
        } else {
            $pdfContent = base64_decode($pdfDocument->fileContent);
            $fileName = $pdfDocument->documentType . '-' . date("d-m-Y", $pdfDocument->updatedOn / 1000) . '.pdf';
        }
        if (!$pdfContent) {
            //todo show error message
            die("no content");
        }

        $gf->markOrderAsDownloaded($orderId, $documentId, $pdfDocument);
        $this->Front()->Plugins()->ViewRenderer()->setNoRender();
        $this->Front()->Plugins()->Json()->setRenderer(false);
        $response = $this->Response();
        $response->setHeader('Content-Type', 'application/pdf');
        $response->setHeader('Content-Description', 'File Transfer');
        $response->setHeader('Content-Disposition', 'attachment; filename="' . $fileName . '"');
        $response->setHeader('Expires', '0');
        $response->setHeader('Cache-Control', 'must-revalidate');
        $response->setHeader('Pragma', 'public');
        $response->setHeader('Content-Transfer-Encoding', 'binary');
        $response->setHeader('Content-Length', strlen($pdfContent));
        $response->sendHeaders();
        echo $pdfContent;
        exit;
    }

    public function deletePostLabelAction()
    {
        $orderId = $this->Request()->getParam('orderId');
        $labelId = $this->Request()->getParam('id');

        if (!$orderId || !$labelId) {
            $this->View()->assign([
                'data' => array('message' => 'Alle Werte sind nicht vorhanden; orderId: $orderId, labelId: $labelId'),
                'success' => false,
            ]);
            return;
        }

        $gf = $this->container->get("post_label_center.greenfield_service");
        $order = Shopware()->Models()->find('Shopware\Models\Order\Order', $orderId);
        $trackingCodes = array_map('trim', explode(';', $order->getTrackingCode()));
        $trackingCodes = array_filter($trackingCodes);
        $orderAttributes = $gf->getOrderAttributes($orderId);
        $orderLabels = json_decode($orderAttributes['post_labels_downloads'], true) ?: [];
        foreach ($orderLabels as $idxL => $orderLabel) {
            if ($orderLabel["documentid"] == $labelId) {
                foreach ($trackingCodes as $idx => $trackingCode) {
                    if ($trackingCode == $orderLabel["label_data"]["tracking_code"]) {
                        unset($trackingCodes[$idx]);
                    }
                }
                unset($orderLabels[$idxL]);
            }
        }
        $trackingCodes = implode('; ', $trackingCodes);
        $order->setTrackingCode($trackingCodes);
        $orderAttributes['post_labels_downloads'] = json_encode($orderLabels);

        try {
            $attributePersister = Shopware()->Container()->get('shopware_attribute.data_persister');
            $attributePersister->persist($orderAttributes, 's_order_attributes', $orderId);
            Shopware()->Models()->flush();
        } catch (\Exception $e) {
            $this->View()->assign([
                'data' => array('message' => $e->getMessage()),
                'success' => false,
            ]);

            return;
        }

        $this->View()->assign([
            'data' => array(),
            'success' => true,
        ]);
    }

    public function getWhitelistedCSRFActions()
    {
        return [
            'openPdf',
            'loadOrderLabels'
        ];
    }
}
