<?php

use Acl\GreenField\Exceptions\GreenFieldResponseException;
use Shopware\Models\Dispatch\Dispatch;
use PostLabelCenter\Models\PostLabelPluginConfiguration;

class Shopware_Controllers_Backend_PostOrder extends Shopware_Controllers_Backend_ExtJs
{

    const PLUGIN_NAME = 'PostLabelCenter';


    public function importMultipleShipmentAction()
    {
        $gf = $this->container->get("post_label_center.greenfield_service");
        $orders = $this->Request()->getPost('orders');
        $orders = json_decode($orders);
        if (!$orders) {
            return false;
        }

        $container = Shopware()->Container();
        $shop = false;

        if ($container->initialized('shop')) {
            $shop = $container->get('shop');
        }

        if (!$shop) {
            /** @var \Shopware\Models\Shop\Repository $shopRepo */
            $shopRepo = $container->get('models')->getRepository(\Shopware\Models\Shop\Shop::class);
            $shop = $shopRepo->getActiveDefault();
        }

        $config = $container->get('shopware.plugin.cached_config_reader')->getByPluginName(self::PLUGIN_NAME, $shop);

        try {
            $response = $gf->importMultipleShipment($orders, $labelType);

            if (get_class($response) === 'Exception') {
                throw $response;
            } else if (is_array($response) && $response['isMessage']) {
                $this->View()->assign([
                    'data' => $response,
                    'success' => false,
                ]);
                return;
            }
            $responses = $response;
            if (!is_array($response) && $response) {
                $responses = [$response];
            }
            $success = true;

        } catch (GreenFieldResponseException $e) {
            // quickfix:  important: these kind of errors should never happen
            // todo
            //$response = json_decode($e->getResponse()->getBody())->error->message;
            $data = array(
                'error' => array(
                    'errorMessage' => json_decode($e->getResponse()->getBody())->errorMsg,
                    'errorMessageExtended' => json_decode($e->getResponse()->getBody())->errorDescription
                )
            );

            $this->View()->assign([
                'data' => $data,
                'success' => false,
            ]);

            return;
        } catch (\Exception $e) {
            $data = array(
                'error' => array(
                    'errorMessage' => $e->getMessage(),
                    'errorMessageExtended' => ''
                )
            );
            $this->View()->assign([
                'data' => $data,
                'success' => false,
            ]);

            return;
        }


        $repository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
        // breaking change, post sometimes return 2 tracking codes for 1 document (there are BOTH labels in document)
        //$shipment = end($response->data->importShipmentResult);
        foreach ($responses as $response) {


            $pdfResponses = $response->data[0]->multiplePdfResponses;
            foreach ($pdfResponses as $pdfResponse) {
                $trackingCodes = [];
                $generatedPdfs = (array)$pdfResponse->generatedPdfs;

                foreach ($generatedPdfs as $orderNumber => $pdfData) {
                    $order = $repository->findOneBy(["number" => $orderNumber]);
                    if (!$order) {
                        //cant find order
                        continue;
                    }

                    $orderId = $order->getId();
                    foreach ($pdfData->importShipmentResult as $shipment) {
                        foreach ($shipment->colloCodeList as $collCodeElement) {
                            $trackingCodes[] = $collCodeElement->code;
                        }
                        if (count($trackingCodes) > 0) {
                            // save view into DB - usability
                            $orderAttributes = $gf->getOrderAttributes($orderId);

                            // todo: check if wanted attribute exists ?
                            if (!$labelsDownloadsList = $orderAttributes['post_labels_downloads']) {
                                $labelsDownloadsList = '[]';
                            }
                            //DO LOGIC
                            // if someone changes order status change only first pf tracking numbers
                            //always use second trackingcode if available
                            if (count($trackingCodes) > 1) {
                                $trackingCode = $trackingCodes[1];
                            } else {
                                $trackingCode = $trackingCodes[0];
                            }
                            $documentId = $pdfData->documentId;
                            $documentType = $pdfData->documentType;
                            $date = new DateTime();
                            $date = $date->format('d.m.Y H:i:s');

                            $isStatusChange ? $source = 'status_change' : $source = 'additional_label';

                            $labelDownload = array(
                                'documentid' => $documentId,
                                'label_data' => array(
                                    'date' => $date,
                                    'type' => $documentType,
                                    'source' => $source,
                                    'downloaded' => false,
                                    'tracking_code' => $trackingCode
                                )
                            );

                            $labelsDownloadsList = json_decode($labelsDownloadsList, true);

                            // if status change replace first tracking number, else just add it
                            $existingTrackingCodes = array_map('trim', explode(';', $order->getTrackingCode()));
                            $existingTrackingCodes[] = $trackingCode;
                            $existingTrackingCodes = implode('; ', $existingTrackingCodes);
                            $order->setTrackingCode($existingTrackingCodes);

                            // manipulate order attribute - first record should always be label,
                            // created by status change
                            if (count($labelsDownloadsList) > 0) {
                                $firstLabel = $labelsDownloadsList[0];
                                if ($firstLabel->label_data->source == 'status_change' && $isStatusChange) {
                                    $labelsDownloadsList[0] = $labelDownload;
                                } else {
                                    $labelsDownloadsList[] = $labelDownload;
                                }
                            } else {
                                $labelsDownloadsList[] = $labelDownload;
                            }

                            $orderAttributes['post_labels_downloads'] = json_encode($labelsDownloadsList);
                            $attributePersister = Shopware()->Container()->get('shopware_attribute.data_persister');
                            $attributePersister->persist($orderAttributes, 's_order_attributes', $orderId);

                            try {
                                Shopware()->Models()->persist($order);
                                Shopware()->Models()->flush();
                            } catch (Exception $e) {
                                $this->View()->assign([
                                    'data' => $e->getMessage(),
                                    'success' => false,
                                ]);
                                return;
                            }
                        }
                    }
                }

            }
        }


        $this->View()->assign([
            'data' => $responses[0]->data,
            'success' => $success,
            'errors' => $errors
        ]);
    }

    public function loadOrderDataAction()
    {
        $orderNumber = $this->Request()->getParam('ordernumber');
        $repository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
        $order = $repository->findOneBy(["number" => $orderNumber]);
        $shopId = $order->getShop()->getId();
        $billing = $order->getBilling();
        $customer = $billing->getCustomer();
        $shipping = $order->getShipping();
        $dispatch = $order->getDispatch();
        $dispatchId = $dispatch->getId();

        $isShipping = false;
        if ($shipping->getStreet() && $shipping->getZipCode() && $shipping->getCity()) {
            $isShipping = true;
        }
        $firstName = $isShipping ? $shipping->getFirstName() : $billing->getFirstName();
        $lastName = $isShipping ? $shipping->getLastName() : $billing->getLastName();
        $company = $isShipping ? $shipping->getCompany() : $billing->getCompany();
        $department = $isShipping ? $shipping->getDepartment() : $billing->getDepartment();
        $salutation = $isShipping ? $shipping->getSalutation() : $billing->getSalutation();

        $street = $isShipping ? $shipping->getStreet() : $billing->getStreet();
        $zip = $isShipping ? $shipping->getZipCode() : $billing->getZipCode();
        $city = $isShipping ? $shipping->getCity() : $billing->getCity();
        $country = $isShipping ? $shipping->getCountry()->getIso() : $billing->getCountry()->getIso();
        $phone = $billing->getPhone();
        $email = $customer->getEmail();
        $invoiceAmount = $order->getInvoiceAmount();
        $products = [];
        $additionalAddressLine1 = $isShipping ? $shipping->getAdditionalAddressLine1() : $billing->getAdditionalAddressLine1();
        $additionalAddressLine2 = $isShipping ? $shipping->getAdditionalAddressLine2() : $billing->getAdditionalAddressLine2();

        foreach ($order->getDetails() as $detail) {

            if ($detail->getMode() !== 0) {
                continue;
            }
            $articleRepository = Shopware()->Models()->getRepository(\Shopware\Models\Article\Detail::class);

            $variant = $articleRepository->findOneBy(['number' => $detail->getArticleNumber()]);
            $product["variant"] = $variant;
            if ($variantAttributes = $variant->getAttribute()) {
                $product["hSTariffNumber"] = $variantAttributes->getHsTariffNumber();
                $product["originCountry"] = $variantAttributes->getOriginCountry();
            }
            $product["quantity"] = $detail->getQuantity();
            $product["name"] = $detail->getArticleName();
            $product["price"] = $detail->getPrice();
            $product["articleNumber"] = $detail->getArticleNumber();
            $product["packageNumber"] = 1;
            $products[] = $product;
            if ($counter === 0) {
                $firstProduct = $variant->getArticle();
            }
            $counter += 1;
        }
        $shippingSetName = "Kein Produkt vorhanden!";
        $options = [];
        // used shipping set
        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\ShippingSet::class);
            $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);
            $dispatchData = Shopware()->Models()->find(Dispatch::class, $dispatchId);
            if ($shippingSet) {
                $features = $shippingSet->getFeatures();
                $options = $shippingSet->getCustomsInformation();
                $shippingSetName = $dispatchData->getName();
            }
        } catch (\Exception $e) {
            // eventual implement error handling
            return $e;
        }

        // config settings
        try {
            $configRepository = Shopware()->Models()->getRepository(PostLabelPluginConfiguration::class);
            $pluginConfig = $configRepository->findAll();
            $fullConfig = $configRepository->findAll();
            if (!$fullConfig || empty($fullConfig) || !isset($fullConfig[0])) {
                return false;
            }
            $fullConfig = $fullConfig[0];
            $availableCountries = $fullConfig->getAvailableCountries();
            $availableCurrencies = $fullConfig->getAvailableCurrencies();
        } catch (\Exception $e) {
            // eventual implement error handling
            return $e;
        }


        $weight = 0;
        foreach ($products as &$product) {

            //no variant for this product
            if (!isset($product["variant"])) {
                continue;
            }
            // find parent price
            if ($product["variant"]->getWeight() <= 0) {
                $mainDetail = $product["variant"]->getArticle()->getMainDetail();
                $weight += $mainDetail->getWeight() * $product["quantity"];
                $product["weight"] = $mainDetail->getWeight();
                continue;
            }
            $product["weight"] = $product["variant"]->getWeight();

            $weight += $product["variant"]->getWeight() * $product["quantity"];
        }

        if ($weight == 0) {
            $weight = 0.1;
        }

        $data = [
            "currency" => "EUR",
            "invoiceAmount" => $invoiceAmount,
            "shippingSetName" => $shippingSetName,
            "options" => $options,
            "shopId" => $shopId,
            "availableCountries" => $availableCountries,
            "availableCurrencies" => $availableCurrencies,
            "address" =>
                [[
                    "firstName" => $firstName,
                    "lastName" => $lastName,
                    "company" => $company,
                    "salutation" => $salutation,
                    "email" => $email,
                    "phone" => $phone,
                    "street" => $street,
                    "city" => $city,
                    "zip" => $zip,
                    "country" => $country,
                    "department" => $department,
                    "additionalAddressLine1" => $additionalAddressLine1,
                    "additionalAddressLine2" => $additionalAddressLine2
                ]],
            "packageData" =>
                [
                    [
                        "weight" => (float)$weight,
                        "options" => $options,
                        "shippingSetName" => $shippingSetName,
                        "returnOptions" => $shippingSetName,
                        "ret" => $shippingSetName,
                    ]
                ],
            "articles" =>
                $products
        ];

        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($data)
        ]);
        return;
    }

    public function importShipmentAction()
    {
        $gf = $this->container->get("post_label_center.greenfield_service");
        $orderNumber = $this->Request()->getPost('ordernumber');
        $isStatusChange = (bool)$this->Request()->getPost('statuschange');
        $labelType = $this->Request()->getPost('labelType');
        $returnAddressValues = json_decode($this->Request()->getPost('returnAddressValues'), true);
        $senderAddressValues = json_decode($this->Request()->getPost('senderAddressValues'), true);
        $senderBankValues = json_decode($this->Request()->getPost('senderBankValues'), true);

        $receiverAddressValues = json_decode($this->Request()->getPost('receiverAddressValues'), true);
        $packageDataValues = json_decode($this->Request()->getPost('packageDataValues'), true);
        $articleDataValues = json_decode($this->Request()->getPost('articleDataValues'), true);
        $customsData = json_decode($this->Request()->getPost('customsData'), true);
        $customsDutyValues = json_decode($this->Request()->getPost('customsDutyValues'), true);
        $documentDocumentDataValues = json_decode($this->Request()->getPost('documentDocumentDataValues'), true);
        $documentArticleDataValues = json_decode($this->Request()->getPost('documentArticleDataValues'), true);
        $orderDataValues = json_decode($this->Request()->getPost('orderDataValues'), true);

        $orderDataForOverwrite = [
            'returnAddress' => $returnAddressValues,
            'senderAddress' => $senderAddressValues,
            'senderBank' => $senderBankValues,
            'receiverAddress' => $receiverAddressValues,
            'packageData' => $packageDataValues,
            'articleData' => $articleDataValues,
            'customsDuty' => $customsDutyValues,
            'documentDocumentData' => $documentDocumentDataValues,
            'documentArticleData' => $documentArticleDataValues,
            'customsData' => $customsData,
            'orderData' => $orderDataValues,

        ];
        $container = Shopware()->Container();
        $shop = false;
        if ($container->initialized('shop')) {
            $shop = $container->get('shop');
        }

        if (!$shop) {
            /** @var \Shopware\Models\Shop\Repository $shopRepo */
            $shopRepo = $container->get('models')->getRepository(\Shopware\Models\Shop\Shop::class);
            $shop = $shopRepo->getActiveDefault();
        }

        $config = $container->get('shopware.plugin.cached_config_reader')->getByPluginName(self::PLUGIN_NAME, $shop);

        if (!$labelType) {
            if ($config['defaultLabelType']) {
                $labelType = $config['defaultLabelType'];
                if ($labelType === "NONE") {
                    return;
                }
            }
        }

        $repository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
        $order = $repository->findOneBy(["number" => $orderNumber]);
        $orderID = $order->getId();

        // prevent multiple imports with status change needed?
        $orderAttributes = $gf->getOrderAttributes($orderID);
        $downloads = isset($orderAttributes['post_labels_downloads']) ? $orderAttributes['post_labels_downloads'] : false;

        if ($isStatusChange && $downloads) {
            $downloads = json_decode($downloads);
            foreach ($downloads as $download) {
                if ($download->source = 'status_change') {
                    // break import, because initial import already exists
                    $data = array(
                        'error' => array(
                            'errorMessage' => 'Label existiert bereits!',
                            'errorMessageExtended' => ''
                        )
                    );

                    $this->View()->assign([
                        'data' => $data,
                        'success' => false,
                    ]);
                }
                return;
            }
        }
        try {
            $response = $gf->importShipment($orderID, $labelType, $orderDataForOverwrite);
            if (get_class($response) === 'Exception') {
                throw $response;
            } else if (is_array($response) && $response['isMessage']) {
                $this->View()->assign([
                    'data' => $response,
                    'success' => false,
                ]);
                return;
            }
            $responses = $response;
            if (!is_array($response) && $response) {
                $responses = [$response];
            }

        } catch (GreenFieldResponseException $e) {
            // quickfix:  important: these kind of errors should never happen
            // todo
            //$response = json_decode($e->getResponse()->getBody())->error->message;
            $response = json_decode($e->getResponse()->getBody());
            if (!$response) {
                $currentError = $e->getResponse()->getBody();
                $currentErrorExtend = $e->getResponse()->getBody();
            } else {
                $currentError = $response->errorMsg;
                $currentErrorExtend = $response->errorDescription;
            }
            $data = array(
                'error' => array(
                    'errorMessage' => $currentError,
                    'errorMessageExtended' => $currentErrorExtend
                )
            );

            $this->View()->assign([
                'data' => $data,
                'success' => false,
            ]);

            return;
        } catch (\Exception $e) {
            $data = array(
                'error' => array(
                    'errorMessage' => $e->getMessage(),
                    'errorMessageExtended' => ''
                )
            );
            $this->View()->assign([
                'data' => $data,
                'success' => false,
            ]);

            return;
        }

        $success = true;
        foreach ($responses as $response) {
            if ($response->getRawResponse()->getResponseCode() != '200') {
                $success = false;
                break;
            }
        }


        // breaking change, post sometimes return 2 tracking codes for 1 document (there are BOTH labels in document)
        //$shipment = end($response->data->importShipmentResult);
        foreach ($responses as $response) {

            $shipments = $response->data[0]->importShipmentResult;
            foreach ($shipments as $shipment) {
                $trackingCodes = [];

                foreach ($shipment->colloCodeList as $collCodeElement) {
                    $trackingCodes[] = $collCodeElement->code;
                    //todo - remove later when multipackage available
                    //      break;
                }
                $trackingCodes = array_filter($trackingCodes);
                if (count($trackingCodes) > 0) {
                    // save view into DB - usability
                    $orderAttributes = $gf->getOrderAttributes($orderID);

                    // todo: check if wanted attribute exists ?
                    if (!$labelsDownloadsList = $orderAttributes['post_labels_downloads']) {
                        $labelsDownloadsList = '[]';
                    }

                    $trackingCode = implode('; ', $trackingCodes);

                    $documentId = $response->data[0]->documentId;
                    $documentType = $response->data[0]->documentType;
                    $date = new DateTime();
                    $date = $date->format('d.m.Y H:i:s');

                    $isStatusChange ? $source = 'status_change' : $source = 'additional_label';

                    $labelDownload = array(
                        'documentid' => $documentId,
                        'label_data' => array(
                            'date' => $date,
                            'type' => $documentType,
                            'source' => $source,
                            'downloaded' => false,
                            'tracking_code' => $trackingCode
                        )
                    );

                    $labelsDownloadsList = json_decode($labelsDownloadsList, true);

                    // if status change replace first tracking number, else just add it
                    $existingTrackingCodes = array_map('trim', explode(';', $order->getTrackingCode()));
                    $existingTrackingCodes = array_filter($existingTrackingCodes);
                    $existingTrackingCodes[] = $trackingCode;
                    $existingTrackingCodes = implode('; ', $existingTrackingCodes);
                    $order->setTrackingCode($existingTrackingCodes);

                    // manipulate order attribute - first record should always be label,
                    // created by status change
                    if (count($labelsDownloadsList) > 0) {
                        $firstLabel = $labelsDownloadsList[0];

                        if ($firstLabel->label_data->source == 'status_change' && $isStatusChange) {
                            $labelsDownloadsList[0] = $labelDownload;
                        } else {
                            $labelsDownloadsList[] = $labelDownload;
                        }
                    } else {
                        $labelsDownloadsList[] = $labelDownload;
                    }

                    $orderAttributes['post_labels_downloads'] = json_encode($labelsDownloadsList);
                    $attributePersister = Shopware()->Container()->get('shopware_attribute.data_persister');
                    $attributePersister->persist($orderAttributes, 's_order_attributes', $orderID);

                    try {
                        Shopware()->Models()->persist($order);
                        Shopware()->Models()->flush();
                    } catch (Exception $e) {
                        $this->View()->assign([
                            'data' => $e->getMessage(),
                            'success' => false,
                        ]);
                        return;
                    }
                }
            }
        }

        $this->View()->assign([
            'data' => $responses,
            'success' => $success,
        ]);
    }

    public function cancelShipmentAction()
    {
        $gf = $this->container->get("post_label_center.greenfield_service");
        $success = true;
        $documentId = $this->Request()->getPost('documentId');
        $orderId = $this->Request()->getPost('orderId');

        if (!$documentId) {
            $this->View()->assign([
                'data' => [],
                'success' => false,
            ]);
            return;
        }

        $order = Shopware()->Models()->find('Shopware\Models\Order\Order', $orderId);
        $orderNumber = $order->getNumber();
        $shopId = $order->getShop()->getId();
        $dispatch = $order->getDispatch();
        $dispatchId = $dispatch->getId();
        $overriddenConfig = false;

        $cancelDocuments = array(
            'documentIds' => array($documentId),
            'orderId' => $orderNumber,
            'shopId' => $shopId
        );

        // used shipping set
        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\ShippingSet::class);
            $shippingSet = $repository->findOneBy(['sPremiumDispatchID' => $dispatchId]);

            if ($shippingSet) {
                $overriddenConfig = $gf->getOverriddenConfig($shippingSet);
            }
        } catch (\Exception $e) {
            // eventual implement error handling
            return $e;
        }

        if ($overriddenConfig) {
            $gf->overrideAuth($overriddenConfig);
        }

        try {
            $response = $gf->cancelShipment($cancelDocuments);

        } catch (GreenFieldResponseException $e) {
            $response = json_decode($e->getResponse()->getBody())->error->message;
            $this->View()->assign([
                'data' => $response,
                'success' => false,
            ]);

            return;

        } catch (\Exception $e) {
            $data = array(
                'error' => array(
                    'errorMessage' => $e->getMessage(),
                    'errorMessageExtended' => ''
                )
            );
            $this->View()->assign([
                'data' => $data,
                'success' => false,
            ]);

            return;
        }

        $rawResponse = $response->getRawResponse();
        $responseBody = json_decode($rawResponse->getBody());

        if ($rawResponse->getResponseCode() != '200') {
            $errors = array(
                'error' => array(
                    'errorMessage' => $responseBody->error->message,
                    'errorMessageExtended' => $responseBody->error->extendedMessage
                )
            );
            $this->View()->assign([
                'data' => $errors,
                'success' => false,
            ]);

            return;
        }

        if (!$responseBody->data[0]->validCanceledShipment) {
            $success = false;
        }

        $this->View()->assign([
            'data' => '',
            'success' => $success,
        ]);
    }

}
