<?php

use Shopware\Components\CSRFWhitelistAware;
use Acl\GreenField\Exceptions\GreenFieldResponseException;
use Acl\GreenField\Exceptions\GreenFieldSDKException;
use Shopware\Models\Shop\Shop;
use PostLabelCenter\Models\PostLabelPluginConfigurationContract;

class Shopware_Controllers_Backend_PostPluginConfiguration extends Shopware_Controllers_Backend_ExtJs implements CSRFWhitelistAware
{


    const LICENSE = "QmFzaWMgWVhWemRISnBZVzVRYjNOMFRtOWtaVHBzUVhaS1FVZFFabVJIVjBzdFFYRmZRalJIT1d0dVluZzVNMHRXWWxOYVFRPT0=";

    public function getPluginConfigurationAction()
    {
        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfiguration::class);
        $configuration = $repository->findAll();
        $data = Shopware()->Models()->toArray($configuration);

        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($data)
        ]);
    }

    /**
     * Get Order Statuses
     *
     * Polls the database for available order statuses and constructs ann array
     * with the first item being the ID of the status, and the second being the
     * name of the status and returns it to the caller.
     *
     * @return array
     */
    protected function getOrderStatusesAction()
    {
        try {
            $db = Shopware()->Db();

            $query = "select id, description from s_core_states where `group` like 'state'";
            $data = $db->fetchAll($query);

        } catch (\Exception $e) {
            $this->get("pluginlogger")
                ->error("Failed getting states.", ["exception" => $e->getMessage()]);

            $this->View()->assign([
                'success' => false,
                'data' => [],
                'total' => 0
            ]);

            return false;
        }

        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($data)
        ]);
    }

    // todo: currently single instance
    public function savePluginConfigurationAction()
    {
        $unitID = $this->Request()->getParam('unitID');

        if (isset($unitID)) {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfiguration::class);

            // should be max one saved configuration
            $allSavedConfigurations = $repository->findAll();
            $savedConfiguration = $repository->findOneBy(['unitID' => $unitID]);

            if ($savedConfiguration) {
                $pluginConfiguration = $savedConfiguration;
            } else {
                // we have one saved but under different guid
                if ($allSavedConfigurations) {
                    $pluginConfiguration = $allSavedConfigurations[0];
                } else {
                    $pluginConfiguration = new \PostLabelCenter\Models\PostLabelPluginConfiguration();
                }
            }


        } else {
            $this->View()->assign([
                "success" => false,
                "error" => 'OrgUnitId is missing'
            ]);
            return;
        }

        $unitID = $this->Request()->getParam('unitID');
        $unitGUID = $this->Request()->getParam('unitGUID');
        $returnTimeMax = $this->Request()->getParam('returnTimeMax');
        $returnReasons = $this->Request()->getParam('returnReasons');
        $clientID = $this->Request()->getParam('clientID');
        $identifier = $this->Request()->getParam('identifier');
        $paperLayout = $this->Request()->getParam('paperLayout');
        $dataImportOnly = $this->Request()->getParam('dataImportOnly') == 'on' ? 1 : 0;
        $returnOrderAllowed = $this->Request()->getParam('returnOrderAllowed') == 'on' ? 1 : 0;
        $infoName = $this->Request()->getParam('infoName');
        $infoNameExtended = $this->Request()->getParam('infoNameExtended');
        $infoPhone = $this->Request()->getParam('infoPhone');
        $infoStreet = $this->Request()->getParam('infoStreet');
        $infoZip = $this->Request()->getParam('infoZip');
        $infoCity = $this->Request()->getParam('infoCity');
        $infoCountry = $this->Request()->getParam('infoCountry');
        $eoriNumber = $this->Request()->getParam('eoriNumber');

        $apiURL = $this->Request()->getParam('apiURL');

        //setting the license for the sanity check
        $license = self::LICENSE;
        $_GET['license'] = $license;

        // todo - validate fields above
        $pluginConfiguration->setUnitID($unitID);
        $pluginConfiguration->setUnitGUID($unitGUID);
        $pluginConfiguration->setLicense($license);
        $pluginConfiguration->setReturnTimeMax($returnTimeMax);
        $pluginConfiguration->setReturnReasons($returnReasons);
        $pluginConfiguration->setClientID($clientID);
        $pluginConfiguration->setApiUrl($apiURL);
        $pluginConfiguration->setIdentifier($identifier);
        $pluginConfiguration->setPaperLayout($paperLayout);
        $pluginConfiguration->setDataImportOnly($dataImportOnly);
        $pluginConfiguration->setReturnOrderAllowed($returnOrderAllowed);
        $pluginConfiguration->setInfoName($infoName);
        $pluginConfiguration->setInfoNameExtended($infoNameExtended);
        $pluginConfiguration->setInfoPhone($infoPhone);
        $pluginConfiguration->setInfoStreet($infoStreet);
        $pluginConfiguration->setInfoZip($infoZip);
        $pluginConfiguration->setInfoCity($infoCity);
        $pluginConfiguration->setInfoCountry($infoCountry);
        $pluginConfiguration->setEoriNumber($eoriNumber);
        $service = $this->container->get("post_label_center.greenfield_service");
        $firstContract = $this->getFirstContract();
        $firstContract ?: null;
        $service->setConfig($pluginConfiguration, $firstContract);
        try {
            $countryResponse = $service->getAllCountries();
            $countries = $countryResponse->data;
            $pluginConfiguration->setAvailableCountries(json_encode($countries));

            $currencyResponse = $service->getAllCurrencies();
            $currencies = $currencyResponse->data;

            $pluginConfiguration->setAvailableCurrencies(json_encode($currencies));
        } catch (GreenFieldResponseException $e) {
        }
        $crudservice = $this->container->get('shopware_attribute.crud_service');
        $countryData = [];
        foreach ($countries as $country) {
            $countryData[] = ["key" => $country->countryISO2, "value" => $country->countryname];
        }
        $crudservice->update(
            's_articles_attributes',
            'originCountry',
            'combobox',
            [
                'arrayStore' => $countryData ?: [],
            ],
            null,
            true,
            0
        );


        try {

            Shopware()->Models()->persist($pluginConfiguration);
            Shopware()->Models()->flush();

            $service->setConfig($pluginConfiguration, $firstContract);
            $service->sanityCheck();

        } catch (Exception $e) {
            $this->View()->assign([
                "success" => false,
                "error" => strval($e->getMessage())
            ]);
            return;
        }


        try {
            $countryResponse = $service->getAllCountries();
            $countries = $countryResponse->data;

            $currencyResponse = $service->getAllCurrencies();
            $currencies = $service->data;

        } catch (GreenFieldResponseException $e) {
        }
        $crudservice = $this->container->get('shopware_attribute.crud_service');
        $countryData = [];
        foreach ($countries as $country) {
            $countryData[] = ["key" => $country->countryISO2, "value" => $country->countryname];
        }
        usort($countryData, [$this, "cmp"]);
        $crudservice->update(
            's_articles_attributes',
            'originCountry',
            'combobox',
            [
                'arrayStore' => $countryData ?: [],
            ],
            null,
            true,
            0
        );
        $metaDataCache = Shopware()->Models()->getConfiguration()->getMetadataCacheImpl();
        $metaDataCache->deleteAll();
        Shopware()->Models()->generateAttributeModels(['s_articles_attributes']);


        $this->View()->assign([
            'success' => true
        ]);
    }

    function cmp($a, $b)
    {
        return strcmp($a["value"], $b["value"]);
    }


    public function getFirstContract()
    {
        $contractRepositry = Shopware()->Models()->getRepository(PostLabelPluginConfigurationContract::class);
        $allContracts = $contractRepositry->findAll();
        if (!$allContracts || empty($allContracts) || !isset($allContracts[0])) {
            return false;
        }
        return $allContracts[0];
    }

    public function getSubShopsAction()
    {
        $repository = Shopware()->Models()->getRepository(Shop::class);
        $shops = $repository->findAll();

        $this->View()->assign([
            'success' => false,
            'data' => [],
            'total' => 0
        ]);

        if (!$shops) {
            return;
        }
        $data = [];
        foreach ($shops as $shop) {
            $data[] = [
                'id' => $shop->getId(),
                'name' => $shop->getName(),
                'title' => $shop->getTitle(),
                'currency' => "EUR"
            ];
        }
        //TODO GET STRING FROM CURRENCY
        //$shop->getCurrency(),

        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($shops)
        ]);

    }

    public function getPluginConfigurationContractsAction()
    {
        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfiguration::class);
        $configurations = $repository->findAll();

        $this->View()->assign([
            'success' => false,
            'data' => [],
            'total' => 0
        ]);

        if (!$configurations) {
            return;
        }

        $configurations = $configurations[0];

        if (!$existingContracts = $configurations->getContractNumbers()) {
            return;
        }

        $contractNumbers = json_decode($existingContracts);

        $this->View()->assign([
            'success' => true,
            'data' => $contractNumbers,
            'total' => count($contractNumbers)
        ]);
    }

    public function deleteAddressAction()
    {
        $params = $this->Request()->getParams();
        $id = $params['id'];
        $success = false;
        $data = array();

        $this->View()->assign([
            'success' => $success,
            'data' => $data,
            'total' => 0
        ]);

        if (!$id) {
            return;
        }

        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationAddress::class);
            $address = $repository->findBy(["id" => $id], null, 1);

            if (!$address) {
                return;
            }

            Shopware()->Models()->remove($address[0]);
            Shopware()->Models()->flush();

            $this->View()->assign([
                'success' => true,
                'data' => $data,
                'total' => 0
            ]);

        } catch (Exception $e) {
            $this->View()->assign(['success' => false, 'errorMsg' => $e->getMessage()]);
        }

    }

    public function getReturnAddressAction()
    {
        $addresses = $this->getAddressByType("return");

        $this->View()->assign([
            'success' => true,
            'data' => $addresses,
            'total' => count($addresses)
        ]);
    }

    public function getSenderAddressAction()
    {
        $addresses = $this->getAddressByType("sender");

        $this->View()->assign([
            'success' => true,
            'data' => $addresses,
            'total' => count($addresses)
        ]);
    }

    public function getAddressByType($addressType)
    {
        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationAddress::class);
        $addresses = $repository->findBy(['addressType' => [$addressType, "both"]]);

        if (!$addresses) {
            return [];
        }
        $data = Shopware()->Models()->toArray($addresses);
        return $data;
    }

    public function getAddressAction()
    {
        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationAddress::class);
        $addresses = $repository->findAll();
        if (!$addresses) {
            $this->View()->assign([
                'success' => false,
                'data' => [],
                'total' => 0
            ]);
            return;
        }
        $data = Shopware()->Models()->toArray($addresses);
        array_walk($data, function (&$value) {
            if ($value["bank"]) {
                $bank = $value["bank"];
                $value["bank"] = $bank["id"];
            }
        });
        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($data)
        ]);
    }

    public function saveAddressAction()
    {
        // get parameter
        $street = $this->Request()->getParam('street');
        $firstName = $this->Request()->getParam('firstName');
        $lastName = $this->Request()->getParam('lastName');
        $city = $this->Request()->getParam('city');
        $zip = $this->Request()->getParam('zip');
        $phone = $this->Request()->getParam('phone');
        $addressType = $this->Request()->getParam('addressType');
        $id = $this->Request()->getParam('id');
        $subShopId = $this->Request()->getParam('subShopId');
        $bankId = $this->Request()->getParam('bankId');
        $addressName = $this->Request()->getParam('addressName');
        $salutation = $this->Request()->getParam('salutation');
        $eoriNumber = $this->Request()->getParam('eoriNumber');
        $department = $this->Request()->getParam('department');
        $company = $this->Request()->getParam('company');

        $addressRepository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationAddress::class);
        $bankRepository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationBank::class);

        $returnResult = array(
            'success' => false,
            'data' => [],
            'total' => 0
        );
        if ($id) {
            $address = $addressRepository->findOneBy(["id" => $id]);
        }

        $bank = $bankRepository->findOneBy(["id" => $bankId]);

        if (!$address) {
            $address = new \PostLabelCenter\Models\PostLabelPluginConfigurationAddress;
        }

        $address->setAddressName($addressName);
        $address->setStreet($street);
        $address->setCity($city);
        $address->setZip($zip);
        $address->setPhone($phone);
        $address->setAddressType($addressType);
        $address->setFirstName($firstName);
        $address->setLastName($lastName);
        //only viable option is AT for the moment
        $address->setCountry("AT");
        $address->setBank($bank);
        $address->setActiveShopId($subShopId);
        $address->setSalutation($salutation);
        $address->setEoriNumber($eoriNumber);
        $address->setCompany($company);
        $address->setDepartment($department);
        $this->View()->assign($returnResult);
        try {
            Shopware()->Models()->persist($address);
            Shopware()->Models()->flush();

        } catch (Exception $e) {
            $this->View()->assign([
                "success" => false,
                "error" => strval($e->getMessage())
            ]);
            return;
        }

        $this->View()->assign([
            'success' => true
        ]);
    }


    public function saveContractNumberAction()
    {
        // get parameter
        $contractNumber = $this->Request()->getParam('contractNumber');
        $unitGUID = $this->Request()->getParam('unitGUID');
        $license = $this->Request()->getParam('license');
        $identifier = $this->Request()->getParam('identifier');

        try {
            $this->container
                ->get("post_label_center.greenfield_service")
                ->overrideAuth(
                    [
                        "orgUnitID" => $contractNumber,
                        "orgUnitGuID" => $unitGUID,
                        "clientId" => $license
                    ]
                )->sanityCheck();

            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfiguration::class);
            $configurations = $repository->findAll();
            $returnResult = array(
                'success' => false,
                'data' => [],
                'total' => 0
            );

            // current version support 1 configuration#
            if (!$configurations) {
                $returnResult['data'] = $errors = array(
                    'error' => array(
                        'errorMessage' => 'No saved configuration found',
                        'errorMessageExtended' => ''
                    )
                );
                $this->View()->assign($returnResult);
                return;
            }

            $configuration = $configurations[0];

            if (!$existingContracts = $configuration->getContractNumbers()) {
                $existingContracts = json_encode(array());
            }

            $existingContracts = json_decode($existingContracts);
            $newContractObject = json_decode(json_encode(
                    array(
                        'contractnumber' => $contractNumber,
                        'unitGUID' => $unitGUID,
                        'license' => $license,
                        'identifier' => $identifier

                    )
                )
            );

            $contractInDb = false;

            foreach ($existingContracts as $existingContract) {
                if ($existingContract->contractnumber == $newContractObject->contractnumber) {
                    $contractInDb = true;
                    break;
                }
            }

            if ($contractInDb) {
                $returnResult['data'] = $errors = array(
                    'error' => array(
                        'errorMessage' => 'Contract already in database',
                        'errorMessageExtended' => ''
                    )
                );
                $this->View()->assign($returnResult);

                return;
            }

            array_push($existingContracts, $newContractObject);
            $configuration->setContractNumbers(json_encode($existingContracts));
            Shopware()->Models()->persist($configuration);
            Shopware()->Models()->flush();

            $this->View()->assign([
                'success' => true,
                'data' => [],
                'total' => 0
            ]);
        } catch (GreenFieldResponseException $e) {
            $response = json_decode($e->getResponse()->getBody());
            $errors = array(
                'error' => array(
                    'errorMessage' => $response->error->message,
                    'errorMessageExtended' => $response->error->extendedMessage
                )
            );
            $this->View()->assign([
                "success" => false,
                "data" => $errors,
                "total" => count($errors)
            ]);
            return;

        } catch (Exception $e) {
            $errors = array(
                'error' => array(
                    'errorMessage' => $e->getMessage(),
                    'errorMessageExtended' => $e->getMessage()
                )
            );
            $this->View()->assign([
                "success" => false,
                "data" => $errors,
                "total" => count($errors)
            ]);
        }
    }

    public function deleteContractAction()
    {
        $params = $this->Request()->getParams();
        $id = $params['id'];
        $success = false;
        $data = array();

        $this->View()->assign([
            'success' => $success,
            'data' => $data,
            'total' => 0
        ]);

        if (!$id) {
            return;
        }

        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationContract::class);
            $contract = $repository->findOneBy(["id" => $id]);

            if (!$contract) {
                return;
            }

            Shopware()->Models()->remove($contract);
            Shopware()->Models()->flush();

            $this->View()->assign([
                'success' => true,
                'data' => $data,
                'total' => 0
            ]);

        } catch (Exception $e) {
            $this->View()->assign(['success' => false, 'errorMsg' => $e->getMessage()]);
        }
    }

    public function deleteBankAction()
    {
        $params = $this->Request()->getParams();
        $id = $params['id'];
        $success = false;
        $data = array();

        $this->View()->assign([
            'success' => $success,
            'data' => $data,
            'total' => 0
        ]);

        if (!$id) {
            return;
        }

        try {
            $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationBank::class);
            $bank = $repository->findBy(["id" => $id], null, 1);

            if (!$bank) {
                return;
            }

            Shopware()->Models()->remove($bank[0]);
            Shopware()->Models()->flush();

            $this->View()->assign([
                'success' => true,
                'data' => $data,
                'total' => 0
            ]);

        } catch (Exception $e) {
            $this->View()->assign(['success' => false, 'errorMsg' => $e->getMessage()]);
        }
    }

    public function getPluginContractAction()
    {
        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationContract::class);
        $contractData = $repository->findAll();

        if (!$contractData) {
            $this->View()->assign([
                'success' => false,
                'data' => [],
                'total' => 0
            ]);
            return;
        }
        $data = Shopware()->Models()->toArray($contractData);


        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($data)
        ]);
    }


    public function getPluginBankAction()
    {
        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationBank::class);
        $bankData = $repository->findAll();

        if (!$bankData) {
            $this->View()->assign([
                'success' => false,
                'data' => [],
                'total' => 0
            ]);
            return;
        }
        $data = Shopware()->Models()->toArray($bankData);


        $this->View()->assign([
            'success' => true,
            'data' => $data,
            'total' => count($data)
        ]);
    }

    public function savePluginContractAction()
    {
        $identifier = $this->Request()->getParam('identifier');
        $license = $this->Request()->getParam('license');
        $contractnumber = $this->Request()->getParam('contractnumber');
        $unitGUID = $this->Request()->getParam('unitGUID');
        $id = $this->Request()->getParam('id');

        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationContract::class);
        $returnResult = array(
            'success' => false,
            'data' => [],
            'total' => 0
        );
        if ($identifier) {
            $contract = $repository->findOneBy(["identifier" => $identifier]);
        }

        if (!$contract) {
            $contract = new \PostLabelCenter\Models\PostLabelPluginConfigurationContract;
        }
        $contract->setIdentifier($identifier);
        $contract->setLicense($license);
        $contract->setContractnumber($contractnumber);
        $contract->setUnitGUID($unitGUID);
        $this->View()->assign($returnResult);


        try {
            Shopware()->Models()->persist($contract);
            Shopware()->Models()->flush();

        } catch (Exception $e) {
            $this->View()->assign([
                "success" => false,
                "error" => strval($e->getMessage())
            ]);
            return;
        }

        $this->View()->assign([
            'success' => true
        ]);
    }

    public function savePluginBankAction()
    {
        $bankAccountOwner = $this->Request()->getParam('bankAccountOwner');
        $bankBic = $this->Request()->getParam('bic');
        $accountIban = $this->Request()->getParam('iban');
        $id = $this->Request()->getParam('id');
        $displayName = $this->Request()->getParam('displayName');

        $repository = Shopware()->Models()->getRepository(\PostLabelCenter\Models\PostLabelPluginConfigurationBank::class);
        $returnResult = array(
            'success' => false,
            'data' => [],
            'total' => 0
        );
        if ($id) {
            $bank = $repository->findOneBy(["id" => $id]);
        }

        if (!$bank) {
            $bank = new \PostLabelCenter\Models\PostLabelPluginConfigurationBank;
        }
        $bank->setBankAccountOwner($bankAccountOwner);
        $bank->setDisplayName($displayName);
        $bank->setBic($bankBic);
        $bank->setIban($accountIban);
        $this->View()->assign($returnResult);


        try {
            Shopware()->Models()->persist($bank);
            Shopware()->Models()->flush();

        } catch (Exception $e) {
            $this->View()->assign([
                "success" => false,
                "error" => strval($e->getMessage())
            ]);
            return;
        }

        $this->View()->assign([
            'success' => true
        ]);
    }

    public function getWhitelistedCSRFActions()
    {
        return [
            'getPluginConfiguration',
            'getPluginConfigurationContracts'
        ];
    }
}
