//{block name="backend/order/post_labels/controller/post_labels"}
Ext.define('Shopware.apps.Order.PostLabels.controller.PostLabels', {
    extend: 'Enlight.app.Controller',

    snippets: {
        deletePostLabel: {
            title: 'Label stornieren',
            message: 'Wollen Sie das Label wirklich stornieren?',
            successTitle: 'Erfolg',
            successMessage: 'Das Label wurde storniert.',
            failureTitle: 'Fehler',
            failureMessage: 'Während des Stornierungvorganges ist ein Fehler aufgetreten.'
        }
    },

    refs: [
        { ref: 'postLabelDocumentsPanel', selector: 'order-post_labels-document-panel' }
    ],

    init: function () {
        var me = this;
        me.control({
            'shipping_post_labels_document_configuration-panel': {
                getAdditionalLabel: me.getAdditionalLabel
            },
            'order-post_labels-document-list': {
                deletePostLabel: me.deletePostLabel
            }
        });
    },


    getAdditionalLabel: function (record, store, editForm, detailToolbar) {
        var me = this;
        me.labelListStore = store;
        me.orderRecord = record;
        me.editForm = editForm;
        me.detailToolbar = detailToolbar;

        if (me.addressList) {
            me.addressList.destroy();
        }


        me.addressList = Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentConfirm', {
            orderRecord: record,
            listeners: {
                scope: me,
                close: me.reloadData
            }
        }).show();

    },
	reloadData: function() {
	var me = this;

        Ext.Ajax.request({
            url: '{url controller=PostTracking action=getTrackingCodes}',
            method: 'POST',
            params: {
                orderNumber: me.orderRecord.get('number')
            },
            success: function(response) {
                var responseText  = Ext.JSON.decode(response.responseText);
                var trackingCode = responseText.data[0];
                me.orderRecord.set('trackingCode', trackingCode);
                me.editForm.loadRecord(me.orderRecord);
                me.detailToolbar.fireEvent('click');
                Ext.ComponentQuery.query('order-list')[0].store.reload();
            }
        });

        if(me.labelListStore) {
	me.labelListStore.load();
	}

    },
    deletePostLabel: function (record, store, order) {

        var me = this;
        var message = me.snippets.deletePostLabel.message,
            title = me.snippets.deletePostLabel.title;

        // we do not just delete - we are polite and ask the user if he is sure.
        Ext.MessageBox.confirm(title, message, function (response) {
            if (response !== 'yes') {
                return;
            }

            me.getPostLabelDocumentsPanel().setLoading(true);

            Ext.Ajax.request({
                url: '{url controller="PostOrder" action="cancelShipment"}',
                params: {
                    'documentId': record.get('id'),
                    'orderId': order.get('id')
                },
                success: function (result) {
                    me.getPostLabelDocumentsPanel().setLoading(false);
                    var operation = Ext.JSON.decode(result.responseText);
                    if (operation.success === true) {
                        Shopware.Notification.createGrowlMessage('Label wurde gelöscht', 'Label wurde erfolgreich gelöscht.');
                        me.removeLabel(record);
                        store.load();
                    } else {
                        Shopware.Notification.createGrowlMessage('Label Löschen Fehler ', 'Fehler beim Löschen von Label!');
                    }
                }
            });

        });
    },

    removeLabel: function (record) {
        var me = this;
        record.destroy({
            callback: function (data, operation) {
                var records = operation.getRecords(),
                    record = records[0],
                    rawData = record.getProxy().getReader().rawData;
                if (operation.success === true) {
                    Shopware.Notification.createGrowlMessage(me.snippets.deletePostLabel.successTitle, me.snippets.deletePostLabel.successMessage, me.snippets.growlMessage);
                } else {
                    Shopware.Notification.createGrowlMessage(me.snippets.deletePostLabel.failureTitle, me.snippets.deletePostLabel.failureMessage /*+ ' ' + rawData.message*/, me.snippets.growlMessage);
                }
            }
        });

    }

});
//{/block}
