//{block name="backend/order/post_labels/view/detail/modal/PostLabelsDocumentArticleData"}
Ext.define('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentArticleData', {
    extend: 'Ext.form.Panel',
    cls: 'shopware-form',
    alias: ' widget.order-post_labels-document-article-data-panel',
    border: false,
	    bodyPadding: '10 10 0 10',

    defaults: {
        anchor: '100%'
    },
    refs: [
        { ref: 'articleList', selector: 'order-post_labels-document-article-data-list'}
    ],
    initComponent: function () {
        var me = this;
        me.store = me.orderDataStore;
	  me.presetOptions =  ["customsoption", "units"];

        me.store.load(
            {
                callback: function (records, operation, success) {
                    var order = me.orderDataStore.first();
                    me.options = JSON.parse(order.get('options'));
                    me.options.forEach(function(option) {
                        for (const [ key, options ] of Object.entries(option)) {
                            var keyName = key.toLowerCase();
                            if(me.presetOptions.includes(keyName)) {
                                var comboBox = Ext.ComponentQuery.query('#' + keyName);
                                me.fillOptions(comboBox[0], options);
                            }
                        }
                    });
                    	 me.fillCountries(order);
                    me.fillCurrencies(order);

			me.loadRecord(records[0]);
                },
            }
        );

        me.items = me.createContainer();
        me.callParent(arguments);
    },
	   fillOptions: function (element, option) {

        if (!option || !element) {
            return;
        }
        var data = [];

        for (const [key, value] of Object.entries(option)) {
	data.push( { "value": key, "name": value } );
        }

        var comboStore = Ext.create('Ext.data.Store', {
            fields: ['name', 'value'],
            data: data
        });
        element.bindStore(comboStore);

    },

    fillCurrencies: function (store) {
        var me = this;
        me.currencies = JSON.parse(store.get('availableCurrencies'));
        var currencyComboBox = Ext.ComponentQuery.query('#currency');
        var currencyData = [];

        for (const [key, value] of Object.entries(me.currencies)) {
            currencyData.push( { "value": value.currency, "name": value.name } );
        }

        var comboStore = Ext.create('Ext.data.Store', {
            fields: ['name', 'value'],
            data: currencyData
        });
        currencyComboBox[0].bindStore(comboStore);

    },

    fillCountries: function (store) {
        var me = this;
	me.countries = JSON.parse(store.get('availableCountries'));
        var countryComboBox = Ext.ComponentQuery.query('#originCountry');
        var countryData = [];

        for (const [key, value] of Object.entries(me.countries)) {
            countryData.push( { "value": value.countryISO2, "name": value.countryname } );
        }

        var comboStore = Ext.create('Ext.data.Store', {
            fields: ['name', 'value'],
            data: countryData
        });
	comboStore.sort("name", 'ASC');
        countryComboBox[0].bindStore(comboStore);

    },
	
    createContainer: function () {
        var leftContainer, rightContainer, me = this;

        leftContainer = Ext.create('Ext.container.Container', {
            columnWidth: 0.5,
            border: false,
            layout: 'anchor',
            defaults: {
                anchor: '100%',
                labelWidth: 155,
                labelStyle: 'font-weight: 700;',
                style: {
                    margin: '0 0 10px'
                },
                store: me.store,
                xtype: 'textfield',
                allowBlank: false
            },
            items: me.createLeftElements()
        });

        rightContainer = Ext.create('Ext.container.Container', {
            columnWidth: 0.5,
            border: false,
            layout: 'anchor',
            items: me.createRightElements()
        });

        var topFieldSet = Ext.create('Ext.form.FieldSet', {
            layout: 'column',
            bodyPadding: 5,
            columnWidth: 1,
            stateful: true,
            title: 'Bestelldaten',
            items: [leftContainer, rightContainer]
        });


        return [topFieldSet, me.createArticleList()];
    },

    createLeftElements: function () {
        var me = this;
        return [ {
            name: 'invoiceAmount',
            fieldLabel: 'Bestellsumme',
            value: me.invoiceAmount,
            anchor: '100%',
            labelWidth: 155,
            labelStyle: 'font-weight: 700;',
            style: {
                margin: '0 0 10px 15px'
            },
            store: me.store,
            xtype: 'textfield',
        }
        ];
    },

    createRightElements: function () {
        return [
            {
                name: 'currency',
                id: 'currency',
                fieldLabel: 'Währung',
                xtype: 'combobox',
                queryMode: 'local',
            	labelWidth: 155,
                displayField: 'name',
                valueField: 'value',
                value: 'EUR',
                style: {
                    margin: '0 0 10px 15px'
                },
                }
        ];
    },
    createArticleList: function () {
        var me = this;
        // field as combination of checked and unchecked checkboxes
        me.articleList =  Ext.create('Ext.grid.Panel', {
            cls: Ext.baseCSSPrefix + 'document-grid',
            autoScroll: true,
            overflow: 'scroll',
            stateful: true,
            minHeight: 50,
            maxHeight: 200,
            store: '',
            id: 'articleList',
            plugins: [
                Ext.create('Ext.grid.plugin.CellEditing', {
                    clicksToEdit: 2,
		     listeners: {
                        afterEdit: function (editor, event) {
				me.articleList.getStore().commitChanges();

                        }
                    }
                })
            ],
            columns: [
                { header: "Paketnummer", dataIndex: 'packageNumber', flex: 0.5,
                    editor: {
                        xtype: 'numberfield',
                        decimalPrecision: 0,
                        minValue: 1
                    },
                },
	                        { header: "Zolloptionen", dataIndex: 'customsOption',
                    editor:  Ext.create('Ext.form.ComboBox', {
                        id: 'customsoption',
                        name: 'customsOption',
                        queryMode: 'local',
                        displayField: 'name',
                        valueField: 'value',
                    }),
			scope: me,
			renderer: me.customsOptionsColumn
                },
{ header: "Einheit", dataIndex: 'units',
                    editor:  Ext.create('Ext.form.ComboBox', {
                        id: 'units',
                        name: 'units',
                        queryMode: 'local',
                        displayField: 'name',
                        valueField: 'value',
                    }),
                        scope: me,
                        renderer: me.articleUnitsColumn
                },
{ header: "Ursprungsland", dataIndex: 'originCountry',
                    editor:  Ext.create('Ext.form.ComboBox', {
                        id: 'originCountry',
                        name: 'originCountry',
                        queryMode: 'local',
                        displayField: 'name',
                        valueField: 'value',

                    }),
                        scope: me,
			minWidth: 150,
                        renderer: me.originCountryColumn
                },


                { header: "Artikelnummer", dataIndex: 'articleNumber' },
  { header: "HSTariffnummer", dataIndex: 'hSTariffNumber',
                    editor: {
                        xtype: 'textfield',
                    },
                },
		{ header: "Bezeichnung", dataIndex: 'name' },
                { header: "Gewicht (kg/Stk)", dataIndex: 'weight',
                    editor: {
                        xtype: 'numberfield',
                        decimalPrecision: 3,
			minValue: 0
                    },
                },
                { header: "Menge", dataIndex: 'quantity', flex: 0.5,
                    editor: {
                        xtype: 'numberfield',
                        decimalPrecision: 0,
                        minValue: 1
                    },
                },
                { header: "Preis", dataIndex: 'price', flex: 0.5 },
                me.createActionColumn()
            ]
        })
        return me.articleList;
    },
	
    customsOptionsColumn: function (value, metaData, record) {
        var me = this;
        if (me.options && value) {
            return me.options[0].customsOption[value];
        }
        return value;
    },
	articleUnitsColumn: function (value, metaData, record) {
        var me = this;

        if (me.options && value) {
            return me.options[0].units[value];
        }
        return value;
    },
 originCountryColumn: function (value, metaData, record) {
        var me = this;

        if (me.countries && value) {
            const currentCountry = me.countries.find(function(countries) {
		return countries.countryISO2 == value;
	});
		return currentCountry.countryname;
        }
        return value;
    },

    createActionColumn: function() {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 90,
            items: [
                me.createArticleDeleteColumn(),
                me.createArticleCopyColumn()
            ]
        });
    },

    createArticleCopyColumn: function() {
        var me = this;
        return {
            iconCls:'sprite-plus-circle-frame',
            tooltip: 'Artikel kopieren',

            handler:function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                var newRecord = record.copy(null);
                newRecord.set('packageNumber', parseInt(newRecord.get('packageNumber')) + 1);
                store.insert(rowIndex + 1, newRecord);
            }
        };
    },

    createArticleDeleteColumn: function() {
        var me = this;
        return {
            iconCls:'sprite-minus-circle-frame',
            tooltip: 'Artikel löschen',

            handler:function (view, rowIndex, colIndex, item) {
                    var message = 'Wollen Sie diesen Artikel wirklich löschen?',
                    title = 'Artikel löschen';
                Ext.MessageBox.confirm(title, message, function (response) {
                    if (response !== 'yes') {
                        return;
                    }
                    var store = view.getStore();
                    store.removeAt(rowIndex);
                });

            }
        };
    },

});
//{/block}
