//{block name="backend/order/post_labels/view/detail/modal/post_labels_document_confirm"}
Ext.define('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentConfirm', {
    extend: 'Enlight.app.Window',
    cls: 'shopware-form',
    alias: ' widget.order-post_labels-document-confirm-panel',
    title: 'Versandetikettendaten prüfen/ändern',
    width: 1000,
    footerButton: false,
    minimizable: false,
    maximizable: false,
    draggable: true,
    autoscroll: true,
    height: 400,
    minHeight: 400,
    stores: [],
    constructor: function() {
        var me = this;

        me.callParent(arguments);

        me.height = undefined;
    },

    initComponent: function() {
        var me = this;
        me.currency = "EUR";
        me.invoiceAmount = 0;
        me.orderDataStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelOrderData');
        me.orderDataStore.getProxy().extraParams.ordernumber = me.orderRecord.get('number');
        me.items = me.createTabPanel();
        me.dockedItems = [{
            xtype: 'toolbar',
            dock: 'bottom',
            ui: 'shopware-ui',
            cls: 'shopware-toolbar',
            items: me.getDockedItems()
        }];

        me.callParent(arguments);
    },

    getDockedItems: function() {
        var me = this,
            dockedItems = [ '->' ];

        var  buttonText = 'Daten Speichern';

        if(!me.skipGenerate) {
            me.labelTypeDropdown = Ext.create('Ext.form.ComboBox', {
                fieldLabel: 'Label Typ',
                name: 'labelType',
                queryMode: 'local',
                displayField: 'name',
                valueField: 'value',
                forceSelection: true,
                allowBlank: false,
                value: "SHIPPING_LABEL",
                store: Ext.create('Ext.data.Store', {
                    fields : ['name', 'value'],
                    data : [
                        { "value":"SHIPPING_LABEL", "name":"Versand Label" },
                        { "value":"RETURN_LABEL", "name":"Retouren Label" },
                        { "value": "BOTH", "name":"Versand- & Retouren Label"}
                    ]
                })
            });
            buttonText = 'Label generieren';
        }
        var createLabelButton = Ext.create('Ext.button.Button', {
            text: buttonText,
            scope: me,
            cls: 'primary',
            handler:function () {
                var data = me.getLabelData();
                if(!me.skipGenerate) {
                    me.generateAdditionalLabel(data);
                } else {
                    me.close();
                }
            }
        });
        dockedItems.push(me.labelTypeDropdown);
        dockedItems.push(createLabelButton);

        return dockedItems;
    },
    registerEvents: function() {
        this.addEvents(
            'generateAdditionalLabel'
        );
    },

    getLabelData: function () {
        var me = this;

        var documentList = Ext.ComponentQuery.query('#documentList');
        var articleList = Ext.ComponentQuery.query('#articleList');
        var documentdataValues =  [];
        var articleDataValues =  [];
        if(documentList.length > 0) {
            var documentStore = documentList[0].getStore();
            documentdataValues = Ext.encode(Ext.pluck(documentStore.data.items, 'data'));
        }

        if(articleList.length > 0) {
            var articleStore = articleList[0].getStore();
            articleDataValues = Ext.encode(Ext.pluck(articleStore.data.items, 'data'));
        }

        return {
            orderNumber: me.orderRecord.get('number'),
            senderAddressValues: me.senderAddress.getForm().getValues(),
            senderBankValues: me.senderBank.getForm().getValues(),
            returnAddressValues: me.returnAddress.getForm().getValues(),
            receiverAddressValues: me.receiverAddress.getForm().getValues(),
            packageDataValues: me.packageData.getForm().getValues(),
	    customsData: me.documentDocumentData.getForm().getValues(),
            documentDocumentDataValues: documentdataValues,
            articleDataValues: articleDataValues,
	    orderDataValues: me.articleData.getForm().getValues()
        };
    },

    generateAdditionalLabel: function (data) {
        var me = this;
        me.mainPanel.setLoading(true);

        var articleList = Ext.ComponentQuery.query('#articleList');
        //var articleDataRecords = articleList[0].getStore().getRange();
       // var articleDataValues = writer.getRecordData(articleDataRecords);

        var labelType = me.labelTypeDropdown.getValue();

        //TODO CHECK FIELD VALIDATION
        if (false) {
            me.mainPanel.setLoading(false);
            Shopware.Notification.createGrowlMessage('Label generierung Fehler ', 'Request Parameter sind nicht vorhanden!');
        }

        Ext.Ajax.request({
            url:'{url controller="PostOrder" action="ImportShipment"}',
            params: {
                'ordernumber': data.orderNumber,
                'senderAddressValues': JSON.stringify(data.senderAddressValues),
                'returnAddressValues': JSON.stringify(data.returnAddressValues),
                'receiverAddressValues': JSON.stringify(data.receiverAddressValues),
                'packageDataValues': JSON.stringify(data.packageDataValues),
                'articleDataValues': JSON.stringify(data.articleDataValues),
                'documentDocumentDataValues': JSON.stringify(data.documentDocumentDataValues),
                'senderBankValues': JSON.stringify(data.senderBankValues),
		'customsData': JSON.stringify(data.customsData),
		'orderDataValues': JSON.stringify(data.orderDataValues),
                'labelType': labelType,
            },
            success: function(result) {
                me.mainPanel.setLoading(false);
                var operation =  Ext.JSON.decode(result.responseText);
                if (operation.success === true) {
                    Shopware.Notification.createGrowlMessage('Label wurde generiert', 'Label wurde erfolgreich generiert.');
                } else {
                    if(operation.data.error) {
                        var title = !operation.data.error ?  'Warnung! Fehler beim generieren von Label' : operation.data.error.errorMessage;
                        Ext.Msg.alert(title, operation.data.error.errorMessageExtended);
                    }
                    Shopware.Notification.createGrowlMessage('Label generierung Fehler ', 'Fehler beim generieren von Label!');
                }
            },
	    failure: function(result) {
                me.mainPanel.setLoading(false);
                    Shopware.Notification.createGrowlMessage('Label generierung Fehler ', 'Fehler beim generieren von Label!');
		}
        });
    },

    createTabPanel: function() {
        var me = this;
        var senderAddressStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelsSenderAddress');
        var returnAddressStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelReturnAddress');
        var senderBankStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelsSenderBank');


        me.senderBank =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentSenderBank', {
            senderBankStore: senderBankStore
        });

        me.returnAddress =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentReturnAddress', {
            returnAddressStore: returnAddressStore
        });
//sender data
        me.senderAddress =   Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentSenderAddress', {
            senderAddressStore: senderAddressStore,
            senderBankStore: senderBankStore,
	    bankForm: me.senderBank,
	    returnForm: me.returnAddress,
	    returnAddressStore: returnAddressStore,
            orderDataStore: me.orderDataStore
        });



        //order data
        me.receiverAddress =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentReceiverAddress', {
            orderDataStore: me.orderDataStore
        });

        me.articleData =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentArticleData', {
            orderDataStore: me.orderDataStore
        });

        me.packageData =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentPackageData', {
            orderDataStore: me.orderDataStore
        });

        /*//customsduty
        me.customsDuty =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentCustomsdutyData', {
            articleDataStore: null
        });*/

        me.documentDocumentData =  Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentDocumentData', {
            orderDataStore: me.orderDataStore
        });

        var senderTab = Ext.create('Ext.form.Panel', {
            title: 'Absenderdaten',
            id: 'senderTab',
            border: false,
            bodyPadding: 0,
            items: [ me.senderAddress, me.senderBank, me.returnAddress ]
        });

        var orderTab = Ext.create('Ext.form.Panel', {
            title: 'Sendungsdaten',
            id: 'orderTab',
            border: false,
            bodyPadding: 0,
            items: [ me.receiverAddress, me.packageData, me.articleData]
        });

        var customsDutyTab = Ext.create('Ext.form.Panel', {
            title: 'Zolldaten',
            border: false,
            id: 'customsDutyTab',
            bodyPadding: 0,
            items: [ me.documentDocumentData ]
        });

        me.mainPanel = Ext.create('Ext.tab.Panel', {
            name: 'main-tab',
            border: false,
            items: [
                orderTab,
                senderTab,
                customsDutyTab
            ],
            listeners: {
                'tabchange': function (tabPanel, tab) {
		 if (tab.id === "customsDutyTab") {
                        var articleList = Ext.ComponentQuery.query('#articleList');
                        var documentList = Ext.ComponentQuery.query('#documentList');
                        var documentStore = documentList[0].getStore();
                        var articleDataStore = articleList[0].getStore();
                        var documentListDataModels = [];
                        var documentListDataDocument = {};
                        if (articleDataStore && documentList) {
                            articleDataStore.each(function (record, id) {
                                if (!documentListDataDocument.hasOwnProperty(record.get('packageNumber'))) {
                                    documentListDataDocument[record.get('packageNumber')] = {
                                        articleCount: 0,
                                        packageNumber: record.get('packageNumber'),
                                        weight: record.get('weight')
                                    };
                                }
                                documentListDataDocument[record.get('packageNumber')] = {
                                    articleCount: parseInt(documentListDataDocument[record.get('packageNumber')].articleCount) + parseInt(record.get('quantity')),
                                    packageNumber: record.get('packageNumber'),
                                    weight: parseFloat(documentListDataDocument[record.get('packageNumber')].weight) * parseInt(record.get('quantity')),
                                };

                            });
                            var documentArray = Object.values(documentListDataDocument);
                            documentArray.forEach(function (article) {
                                var documentRecordIndex = documentStore.find('packageNumber', article.packageNumber);
                                var documentRecord = documentStore.getAt(documentRecordIndex);

                                var documentNumber = '';
                                var documentType = 0;
                                var customsOption = 0;

                                if(documentRecord) {
                                    documentNumber = documentRecord.get('documentNumber');
                                    documentType = documentRecord.get('documentType');
                                    customsOption = documentRecord.get('customsOption');
                                }

                                var documentListDataModel = Ext.create('Shopware.apps.Order.PostLabels.model.PostLabelDocumentData', {
                                    documentNumber: documentNumber,
                                    documentType: documentType,
                                    customsOption: customsOption,
                                    packageNumber: article.packageNumber,
                                    numberOfArticles: article.articleCount,
                                    weight: article.weight
                                });

                                documentListDataModels.push(documentListDataModel);

                            });

                            var documentListStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelDocumentData', {
                                data: documentListDataModels
                            });
                            documentList[0].reconfigure(documentListStore);
                        }
                    }
                }
            }
        });
        return me.mainPanel;
    }
});
//{/block}
