//{block name="backend/order/post_labels/view/detail/modal/PostLabelsDocumentCustomsdutyData"}
Ext.define('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentCustomsdutyData', {
    extend:'Ext.form.Panel',
    cls: 'shopware-form',
    alias: 'widget.order-post_labels-document-customsduty-data-panel',
    border: false,
    defaults: {
        anchor: '100%'
    },



    initComponent: function() {
        var me = this;
        me.store = me.documentStore;
        me.presetOptions =  ["returnway", "customsoption", "shipmentdocumententry", "returnoptions"];
        me.orderDataStore.load({
            callback: function() {
                var order = me.orderDataStore.first();
                me.options = JSON.parse(order.get('options'));
                me.options.forEach(function(option) {
                    for (const [ key, options ] of Object.entries(option)) {
                        var keyName = key.toLowerCase();
                        if(me.presetOptions.includes(keyName)) {
                            var comboBox = Ext.ComponentQuery.query('#' + keyName);
                            me.fillOptions(comboBox[0], options);
                        }
                    }
                });
	    me.loadRecord(me.record);
        me.fillDocumentList();
            }

        });
	

        me.items = me.createContainer();
        //me.fillDocumentList();
        me.callParent(arguments);
    },

    fillOptions: function (element, option) {
        var me = this;

        if (!option || !element) {
            return;
        }
        var data = [];

        for (const [key, value] of Object.entries(option)) {
            data.push( { "value": key, "name": value } );
        }

        var comboStore = Ext.create('Ext.data.Store', {
            fields: ['name', 'value'],
            data: data
        });
        element.bindStore(comboStore);

    },

    splitBySeparator: function(data) {
        if(!data) {
            return [];
        }
        if(data.includes(',')) {
            return data.split(',');
        }
        return [data];
    },

    fillDocumentList: function() {
        var me = this;

        if (me.record.get('documentType')) {
            var documentTypes = me.splitBySeparator(me.record.get('documentType'));
            var documentNumber = me.splitBySeparator(me.record.get('documentNumber'));
            var documentListDataModels = [];

            documentTypes.forEach(function (documentType, index) {
                var documentListDataModel = Ext.create('Shopware.apps.Order.PostLabels.model.PostLabelDocumentDataDetails', {
                    documentType: documentTypes[index],
                    documentNumber: documentNumber[index],
                });
                documentListDataModels.push(documentListDataModel);

            });

            var documentListStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelDocumentDataDetails', {
                data: documentListDataModels
            });

            me.documentDetailList.reconfigure(documentListStore);
        }

    },

    createContainer: function () {
        var leftContainer, me = this;

        leftContainer = Ext.create('Ext.container.Container', {
            columnWidth: 0.5,
            border: false,
            layout: 'anchor',
            defaults: {
                anchor: '100%',
                labelWidth: 155,
                labelStyle: 'font-weight: 700;',
                style: {
                    margin: '0 0 10px'
                },
                store: me.store,
                xtype: 'textfield',
                allowBlank : false
            },
            items: me.createLeftElements()
        });

        var topFieldSet = Ext.create('Ext.form.FieldSet', {
            layout: 'anchor',
            bodyPadding: 5,
            columnWidth: 1,
            stateful: true,
            title: 'Zollsendungsdaten',
            items: [leftContainer]
        });

        return [topFieldSet];
    },

    createDocumentList: function () {
        var me = this;
        me.documentDetailList =  Ext.create('Ext.grid.Panel', {
            cls: Ext.baseCSSPrefix + 'document-grid',
            autoScroll: true,
            overflow: 'scroll',
            stateful: true,
            minHeight: 50,
            store: me.articleStore,
            dockedItems: me.getToolbar(),
            plugins: [
                Ext.create('Ext.grid.plugin.CellEditing', {
                    clicksToEdit: 1
                })
            ],
            columns: [
                { header: "Dokumentennummer", dataIndex: 'documentNumber',
                    editor: {
                        xtype: 'textfield',
                    },
                },
		                { header: "Menge", dataIndex: 'quantity',
                    editor: {
			 xtype: 'numberfield',
                        decimalPrecision: 0,
                        minValue: 1
                    },
                },
                { header: "Dokumentenart", dataIndex: 'documentType',
                    editor:  Ext.create('Ext.form.ComboBox', {
                        id: 'shipmentdocumententry',
                        name: 'documentType',
                        queryMode: 'local',
                        displayField: 'name',
                        valueField: 'value',
                    }),
		                renderer: me.documentColumn,
				scope: me,

                },
                me.createActionColumn()
            ]
        })

        return me.documentDetailList;
    },
	    /**
 *      * Renderer function
 *           * @param value
 *                * @param metaData
 *                     * @param record
 *                          */
    documentColumn: function (value, metaData, record) {
        var me = this;

	if(me.options) {
	return me.options[0].shipmentDocumentEntry[value];

	}
	return value;
    },
    createActionColumn: function() {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 60,
            items: [
                me.createDocumentCopy(),

            ]
        });
    },

    createDocumentCopy: function() {
        var me = this;
        return {
            iconCls:'sprite-minus-circle-frame',
            tooltip: 'Dokument entfernen',

            handler:function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.removeAt(rowIndex);
            }
        };
    },

    getToolbar: function() {
        var me = this;

        var addAddressButton = Ext.create('Ext.button.Button', {
            iconCls: 'sprite-plus-circle',
            text: 'hinzufügen',
            scope: me,
            handler:function () {
                me.addDocumentRow();
            }
        });

        return Ext.create('Ext.toolbar.Toolbar', {
            dock:'top',
            ui: 'shopware-ui',
            items:[
                addAddressButton,
                '->',]
        });
    },
	    addDocumentRow: function () {
        var me = this;
        var documentStore = null;
        if(me.documentDetailList) {
            documentStore = me.documentDetailList.getStore();
        }

        var documentListDataModel = Ext.create('Shopware.apps.Order.PostLabels.model.PostLabelDocumentDataDetails', {
            documentType: null,
            documentNumber: '',
        });
        if(!documentStore) {
            documentStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelDocumentDataDetails', {
                data: [ documentListDataModel ]
            });
        } else {
            documentStore.add(documentListDataModel);
        }

        me.documentDetailList.reconfigure(documentStore);

    },

    createLeftElements: function () {
        var me = this;
        return [
            me.createDocumentList()
        ];
    },
});
//{/block}
