//{block name="backend/order/post_labels/view/detail/modal/PostLabelsDocumentDocumentData"}
Ext.define('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentDocumentData', {
    extend:'Ext.form.Panel',
    cls: 'shopware-form',
        bodyPadding: 10,

    alias: 'widget.order-post_labels-document-document-data-panel',
    border: false,
    defaults: {
        anchor: '100%'
    },
    initComponent: function() {
        var me = this;
	        me.presetOptions =  ["returnway", "customsoption", "shipmentdocumententry", "returnoptions"];
        me.orderDataStore.load({
            callback: function() {
                var order = me.orderDataStore.first();
                me.options = JSON.parse(order.get('options'));
                me.options.forEach(function(option) {
                    for (const [ key, options ] of Object.entries(option)) {
                        var keyName = key.toLowerCase();
                        if(me.presetOptions.includes(keyName)) {
                            var comboBox = Ext.ComponentQuery.query('#' + keyName);
                            me.fillOptions(comboBox[0], options);
                        }
                    }
                });
                me.loadRecord(me.record);
            }
        });
        me.items = me.createContainer();
        me.callParent(arguments);
    },
	createContainer: function () {
        var me = this;
        return [ {
            name: 'customsDescription',
            fieldLabel: 'Zollbeschreibung',
            xtype: 'textareafield',
        },
            {
                name: 'returnOption',
                id: 'returnoptions',
                fieldLabel: 'Rücksende-Option',
                xtype: 'combobox',
                queryMode: 'local',
                displayField: 'name',
                valueField: 'value',
                store: ''
            },
            {
                name: 'returnWay',
                id: 'returnway',
                fieldLabel: 'Rücksende-Weg',
                xtype: 'combobox',
                queryMode: 'local',
                displayField: 'name',
                valueField: 'value',
                store: ''
            },
            me.createDocumentList()];
    },
	   fillOptions: function (element, option) {
        var me = this;

        if (!option || !element) {
            return;
        }
        var data = [];

        for (const [key, value] of Object.entries(option)) {
            data.push( { "value": key, "name": value } );
        }

        var comboStore = Ext.create('Ext.data.Store', {
            fields: ['name', 'value'],
            data: data
        });
        element.bindStore(comboStore);

    },
    getToolbar: function() {
        var me = this;

        var addAddressButton = Ext.create('Ext.button.Button', {
            iconCls: 'sprite-plus-circle',
            text: 'hinzufügen',
            scope: me,
            handler:function () {
                me.showAddressDetails();
            }
        });

        return Ext.create('Ext.toolbar.Toolbar', {
            dock:'top',
            ui: 'shopware-ui',
            items:[
                addAddressButton,
                '->',]
        });
    },

    createActionColumn: function() {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 60,
            items: [
                me.createEditDocumentColumn(),

            ]
        });
    },


    createEditDocumentColumn: function() {
        var me = this;
        return {
            iconCls:'sprite-pencil',
            tooltip: 'Zolldaten bearbeiten',

            handler:function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.showCustomsWindow(record, rowIndex);
            }
        };
    },

    showCustomsWindow: function (record, index) {
        var me = this;

        //make sure only one instance is open at the same time
        if (me.customsDataWindow) {
            me.customsDataWindow.destroy();
        }
        me.customsDataWindow = Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentCustomsDataWindow', {
            orderDataStore: me.orderDataStore,
            documentListStore: me.store,
            record: record,
            index: index,
        });
        me.customsDataWindow.show();


    },
	    /**
 *      * Renderer function
 *           * @param value
 *                * @param metaData
 *                     * @param record
 *                          */
    documentNumberColumn: function (value, metaData, record) {
        var me = this
	if(me.options && value) {

	  var displayValue = "";
	if(value.includes(',')) {

	var valueArray = value.split(',');

	   valueArray.forEach(function(currentValue) {
	  	displayValue += me.options[0].shipmentDocumentEntry[currentValue] + ", ";
	   });
	} else {
		return me.options[0].shipmentDocumentEntry[value];
	
	} 
	
	return displayValue;
	}
	if(!value) {
	return "";
	}
	return value;


    },

    createDocumentList: function () {
        var me = this;
        // field as combination of checked and unchecked checkboxes
        return Ext.create('Ext.grid.Panel', {
            alias: 'widget.post_plugin_configuration-address-list',
            cls: Ext.baseCSSPrefix + 'document-grid',
            autoScroll: true,
            id: 'documentList',
            overflow: 'scroll',
            stateful: true,
            minHeight: 50,
            maxHeight: 200,
            store: me.store,
            columns: [
                { header: "Paketnummer", dataIndex: 'packageNumber' },
                { header: "Anzahl Artikel", dataIndex: 'numberOfArticles', hidden: true },
                { header: "Gewicht", dataIndex: 'weight', hidden: true },
	        { header: "Dokumentenart", dataIndex: 'documentType', scope: me,  renderer: me.documentNumberColumn },
	        { header: "Dokumentennummer", scope: me, dataIndex: 'documentNumber' },

                me.createActionColumn()

            ]
        })
    },
});
//{/block}
