//{block name="backend/order/post_labels/view/list/batch_order_list"}
Ext.define('Shopware.apps.Order.PostLabels.view.list.BatchOrderList', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.batch-order-list',
    cls: Ext.baseCSSPrefix + 'document-grid',
    autoScroll: true,
    overflow: 'scroll',
    stateful: true,
    minHeight: 50,
    maxHeight: 150,
    snippets: {
        columns: {
            number: 'Bestellnummer',
            targetCountry: 'Zielland',
            status: 'Status',
            createExportDocument: 'Export-Dokumente erstellen',
        }
    },
    initComponent: function () {
        var me = this;
        me.store = me.batchOrderStore;
        me.overwriteLabelData = {};
        me.title = me.customTitle;
        me.dockedItems = [ me.getToolbar() ];
        me.selectedIndex = 0;
        me.columns = [
            { header: me.snippets.columns.number, dataIndex: 'number', flex: 1 },
            { header: me.snippets.columns.targetCountry, dataIndex: 'targetCountry', flex: 1 },
            { header: me.snippets.columns.status, dataIndex: 'status', flex: 1 },
            { header: me.snippets.columns.createExportDocument, dataIndex: 'createExportDocument'},
            me.showAction ? me.createActionColumn(): []
        ];

        me.callParent(arguments);
    },

    getToolbar: function () {
        var me = this;

        return Ext.create('Ext.toolbar.Toolbar', {
            dock: 'top',
            ui: 'shopware-ui',
            items: [
                '->',]
        });
    },

    showCreateLabelModal: function (record,) {
        var me = this;

        if(me.confirmDataWindow) {
            me.confirmDataWindow.destroy();
        }
        me.confirmDataWindow = Ext.create('Shopware.apps.Order.PostLabels.view.detail.modal.PostLabelsDocumentConfirm', {
            orderRecord: record,
            skipGenerate: true,
            listeners:{
                close:me.saveData,
                scope: me
            }
        });


        me.confirmDataWindow.show();
    },

    createLabelModal: function () {
        var me = this;
        return {
            iconCls: 'sprite-pencil',
            tooltip: 'Bestellungsdaten bearbeiten',

            handler: function (view, rowIndex, colIndex, item) {
                var store = view.getStore();
                var record = me.selection[rowIndex];
                me.selectedIndex = rowIndex;
                me.showCreateLabelModal(record, rowIndex);
            }
        };
    },

    createActionColumn: function () {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 60,
            items: [
                me.createLabelModal(),
                me.removeOrderRecord()
            ]
        });
    },

    saveData: function(confirmWindow, rowIndex) {
        var me = this;

        var data = confirmWindow.getLabelData();
        if(!me.overwriteLabelData) {
            me.overwriteLabelData = {};
        }
        data = me.formatData(data);
        me.overwriteLabelData[data.orderNumber] = data;
        if(Ext.JSON.decode(data.documentDocumentDataValues).length > 0) {
            var selectedStore = me.batchOrderStore.getAt(me.selectedIndex);
            selectedStore.set("createExportDocument", "vorhanden");
        }
    },

    formatData: function (data) {
        Object.keys(data).forEach(function (key) {
            if (data.hasOwnProperty(key)) {
                var currentData = data[key];
                if (typeof currentData === 'object' && currentData !== null) {
                    data[key] = JSON.stringify(currentData);
                }
            }
        });
        return data;
    },



    removeOrderRecord: function () {
        var me = this;
        return {
            iconCls: 'sprite-minus-circle-frame',
            tooltip: 'Bestellung entfernen?',

            handler: function (view, rowIndex, colIndex, item) {
                var message = 'Wollen Sie diese Bestellung wirklich entfernen?',
                    title = 'Bestellung entfernen';
                Ext.MessageBox.confirm(title, message, function (response) {
                    if (response !== 'yes') {
                        return;
                    }
                    var store = view.getStore();
                    store.removeAt(rowIndex);
                });
            }
        };
    },

});
//{/block}

