//{block name="backend/order/post_labels/view/list/batch_progress_window"}
Ext.define('Shopware.apps.Order.PostLabels.view.list.BatchProgressWindow', {
    extend: 'Enlight.app.Window',
    title: 'Versandetiketten generieren',
    alias: 'widget.post_label_progress_window',
    minHeight: '100px',
    maxHeight: 600,
    width: 600,
    buttonAlign : 'center',
    layout: 'fit',
    stateful : true,
    overflow: 'scroll',

    initComponent: function() {
        var me = this;


        me.totalCount = me.selection.length;
        me.doneLabelsReturn = [];
        me.doneLabelsShipping = [];
        me.openTasks = me.selection;
	        me.progressTask = [];

        me.items = me.createPanel();
        me.dockedItems = [{
            xtype: 'toolbar',
            dock: 'bottom',
            ui: 'shopware-ui',
            cls: 'shopware-toolbar',
            items: me.getDockedItems()
        }];

        me.callParent(arguments);
    },

    getDockedItems: function() {
        var me = this,
            dockedItems = [ '->' ];

        me.labelTypeDropdown = Ext.create('Ext.form.ComboBox', {
            fieldLabel: 'Label Typ',
            name: 'labelType',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'value',
            forceSelection: true,
            allowBlank: false,
            value: "SHIPPING_LABEL",
            store: Ext.create('Ext.data.Store', {
                fields : ['name', 'value'],
                data : [
                    { "value":"SHIPPING_LABEL", "name":"Versand Label" },
                    { "value":"RETURN_LABEL", "name":"Retouren Label" },
                    { "value": "BOTH", "name":"Versand- & Retouren Label"}
                ]
            })
        });

	var createLabelButton = Ext.create('Ext.button.Button', {
            text: 'Label generieren',
            scope: me,
            cls: 'primary',
            handler: function () {
                var overwriteData = me.orderList.overwriteLabelData;
                me.mainPanel.setLoading(true);
                var labelType = me.labelTypeDropdown.getValue();
                var data = [];
                me.openTasks.forEach(function (record, id) {
                    var orderData;
                    if (overwriteData.hasOwnProperty(record.get('number'))) {
                        orderData = overwriteData[record.get('number')];
                        orderData.orderNumber = record.get('number');
                        orderData.labelType = labelType;
                    } else {
                        orderData = {
                            'ordernumber': record.get('number'),
                            'labelType': labelType
                        };
                    }
                    data.push(orderData);
                });

                me.generateLabel(JSON.stringify(data), labelType);
            }
        });
	 /*var downloadLabelButton = Ext.create('Ext.button.Button', {
            text: 'Label runterladen',
            scope: me,
            cls: 'primary',
            handler:function () {
		window.open('/LABELS.zip');
            }
        }); */

	 var downloadLabelButton = Ext.create('Ext.button.Button', {
            text: 'Label herunterladen',
            scope: me,
            cls: 'primary',
            handler:function () {
                var params = [];
                me.selection.forEach(function (record, id) {
                    params.push(record.get('number'));
                });

                var form = Ext.create('Ext.form.Panel', {
                    'standardSubmit': true,
                    'url': '{url controller="PostLabels" action="loadOrderLabelsZip"}',
                    'method': 'POST',
                });
                form.submit({
                    'params': { "orderNumbers[]": params},
		     'target': '_blank'
                })
            }
        });


        dockedItems.push(me.labelTypeDropdown, createLabelButton);
        dockedItems.push(downloadLabelButton);

        return dockedItems;
    },


    updateProgressBar: function() {
        var me = this;
        var index = me.totalCount - me.openTasks.length;
        me.progressBar.updateProgress(
            index / me.totalCount,
            index + '/' + me.totalCount + ' Bestellungen erfolgreich bearbeitet (' + me.doneLabelsShipping.length + ' Versand-Label,' + me.doneLabelsReturn.length + ' Retour-Label erstellt)'
        );
    },

    createPanel: function() {
        var me = this;
        var orderStatusStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelOrderStatus');
        orderStatusStore.load();
        me.progressBar = Ext.create('Ext.ProgressBar', {
            animate: true,
            text: '0 Bestellungen erfolgreich bearbeitet (0 Versand-Label, 0 Retour-Label erstellt)',
            value: 0,
            height: 20,
            margin: '15 0 0'
        });

        me.statusDropdown = Ext.create('Ext.form.ComboBox', {
            fieldLabel: 'Neuer Bestellstatus',
            name: 'orderStatus',
            queryMode: 'local',
            displayField: 'description',
            valueField: 'id',
            store: orderStatusStore,
        });

        var checkBox = {
            xtype: 'checkbox',
            fieldLabel: 'Status-E-Mails automatisch senden',
            uncheckedValue: false,
            name: 'sendMails'
        };

        me.orderListData = [];
        me.orderListDataModel = [];

        me.openTasks.forEach(function (record, id) {

            var documentListDataModel = Ext.create('Shopware.apps.Order.PostLabels.model.PostLabelBatchOrder', {
                number: record.get('number'),
                targetCountry: record.getShipping().first().raw.country.iso,
                status: record.getOrderStatus().first().get('description'),
                createExportDocument: "nicht vorhanden"
            });
            me.orderListDataModel.push(documentListDataModel);

        });

        me.batchOrderStore = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelBatchOrder', {
            data: me.orderListDataModel
        });

        me.batchOrderStoreSuccess = Ext.create('Shopware.apps.Order.PostLabels.store.PostLabelBatchOrder', {
            data: []
        });

        me.orderList =  Ext.create('Shopware.apps.Order.PostLabels.view.list.BatchOrderList', {
            batchOrderStore: me.batchOrderStore,
            selection: me.selection,
            showAction: true,
            customTitle: 'Auftragsliste'
        });

        me.orderListSuccess =  Ext.create('Shopware.apps.Order.PostLabels.view.list.BatchOrderList', {
            batchOrderStore: me.batchOrderStoreSuccess,
            showAction: false,
            customTitle: 'Label erfolgreich erstellt'
        });

        var topContainer = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: '',
            defaults: {
                anchor: '100%',
                labelWidth: 155,
                labelStyle: 'font-weight: 700;',
                style: {
                    margin: '0 0 10px'
                },
                store: me.store,
                xtype: 'textfield',
                allowBlank: false
            },
            items: [ me.statusDropdown, checkBox]
        });

        var middleContainer = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'anchor',
            defaults: {
                anchor: '100%',
                labelWidth: 155,
                labelStyle: 'font-weight: 700;',
                style: {
                    margin: '0 0 10px'
                },
                store: me.store,
                xtype: 'textfield',
                allowBlank: false
            },
            items: [ me.orderList,  me.orderListSuccess ]
        });


        me.mainPanel = Ext.create('Ext.form.Panel', {
            id: 'batchPanel',
            border: false,
            bodyPadding: 10,
            items: [ topContainer, middleContainer ]
        });


        return me.mainPanel;
    },


    updateOrderSuccessList: function (record) {
        var me = this;
        me.batchOrderStoreSuccess.add(record);

    },

    triggerShopwareBatchUpdate: function () {
        var me = this;
        var data = me.mainPanel.getValues();
        var store = Ext.create('Shopware.apps.Order.store.Batch');
        store.getProxy().extraParams = {
            docType: null,
            mode: data.orderStatus,
            forceTaxCheck: 1,
            displayDate: new Date(),
            deliveryDate: new Date(),
            autoSend: data.sendMails === 'on',
            addAttachments: null
        };
        //prepare orders
        Ext.each(me.selection, function (order) {
            order.set('status', data.orderStatus);
            order.setDirty();
        });
        store.add(me.selection);
        store.sync();

    },


    resetProgressBar: function() {
        var me = this;
        me.totalCount = me.selection.length;
        me.doneLabelsReturn = [];
        me.doneLabelsShipping = [];
        me.openTasks = me.selection;
        me.progressBar = Ext.create('Ext.ProgressBar', {
            animate: true,
            text: '0/0 Bestellungen erfolgreich bearbeitet (0 Versand-Label, 0 Retour-Label erstellt)',
            value: 0,
            height: 20,
            margin: '15 0 0'
        });
    },

    generateLabel: function(params, labelType) {
        var me = this;

        Ext.Ajax.request({
            url: '{url controller="PostOrder" action="importMultipleShipment"}',
            params: { orders: params},
            success: function (result) {
                var operation = Ext.JSON.decode(result.responseText);
		
		if(Array.isArray(operation.data)) {
		operation.data.forEach(function(currentData) {
			if(currentData.errorResponseList && currentData.errorResponseList.length > 0) {
			currentData.errorResponseList.forEach(function(fails) {
                    Shopware.Notification.createGrowlMessage('Label-Generierung Fehler ', 'Fehler beim Generieren von Label für die Bestellung: ' + fails.orderId);
			});

			}
		
			if(currentData.multiplePdfResponses && currentData.multiplePdfResponses.length > 0) {
			currentData.multiplePdfResponses.forEach(function(success) {
				Object.keys(success.generatedPdfs).forEach(function(key) {
				var currentRecord = me.batchOrderStore.findRecord("number", key);
				if(currentRecord) {
			
				me.openTasks = me.openTasks.filter(function(task) {
					return task.data.number !== currentRecord.raw.number;
				});
				me.batchOrderStore.remove(currentRecord);
				me.batchOrderStoreSuccess.add(currentRecord.raw);
                    Shopware.Notification.createGrowlMessage('Label wurde generiert', 'Label für die Bestellung: ' + currentRecord.raw.number + ' wurde erfolgreich generiert.');
				}
				});

			});
	
}
			});
	} else {
                    Shopware.Notification.createGrowlMessage('Label-Generierung Fehler ', 'Es sind Fehler beim Generieren aufgetreten.');
	}
		
                 me.mainPanel.setLoading(false);
            },
		failure: function (result) {
			console.log(result);
			console.log("failure");
                    Shopware.Notification.createGrowlMessage('Label-Generierung Fehler ', 'Es sind Fehler beim Generieren . Zeitüberschreitung...');
                 me.mainPanel.setLoading(false);
		}
        });
    }

});
//{/block}
