//{block name="backend/post_plugin_configuration/controller/address"}
Ext.define('Shopware.apps.PostPluginConfiguration.controller.Address', {
    extend: 'Enlight.app.Controller',

    refs: [
        { ref: 'window', selector: 'post_plugin_configuration-main-window'},
        { ref: 'addressForm', selector: 'post_plugin_address-panel'}
    ],

    init: function () {
        var me = this;
        me.control({
            'post_plugin_configuration-address-list': {
                deleteAddress: me.onDeleteAddress,
                loadAddress: me.onLoadAddress
            },
            'post_plugin_address-panel-window': {
                addAddress: me.onAddAddress
            },
        });
        me.callParent(arguments);
    },

    onLoadAddress: function (record) {
        var me = this;

        var formComponent = me.getAddressForm();
        var addressForm = formComponent.getForm();
        addressForm.loadRecord(record);

    },
    onDeleteAddress: function (record) {

        var me = this,
            store = me.subApplication.getStore('Address'),
            message = 'Wollen Sie die Adresse wirklich löschen?',
            title = 'Adresse löschen';

        // we do not just delete - we are polite and ask the user if he is sure.
        Ext.MessageBox.confirm(title, message, function (response) {
            if (response !== 'yes') {
                return;
            }
            record.destroy({
                callback: function (data, operation) {
                    var records = operation.getRecords(),
                        record = records[0],
                        rawData = record.getProxy().getReader().rawData;

                    if (operation.success === true) {
                        //Shopware.Notification.createGrowlMessage(me.snippets.deleteOrder.successTitle, me.snippets.deleteOrder.successMessage, me.snippets.growlMessage);
                    } else {
                        //Shopware.Notification.createGrowlMessage(me.snippets.deleteOrder.failureTitle, me.snippets.deleteOrder.failureMessage + ' ' + rawData.message, me.snippets.growlMessage);
                    }
                    store.load();
                }
            });
        });
    },

    onAddAddress: function (record) {
        var me = this;

        var formComponent = me.getAddressForm();
        var addressForm = formComponent.getForm();
        var id = false;
        if(record) {
            id = record.get('id');
        }
        var street = addressForm.findField('street').getValue();
        var zip = addressForm.findField('zip').getValue();
        var city = addressForm.findField('city').getValue();
        var phone = addressForm.findField('phone').getValue();
        var firstName = addressForm.findField('firstName').getValue();
        var lastName = addressForm.findField('lastName').getValue();
        var addressType = addressForm.findField('addressType').getValue();
        var subShopId = addressForm.findField('subShopId').getValue();
        var bankId = addressForm.findField('bankId').getValue();
        var addressName = addressForm.findField('addressName').getValue();
	var salutation = addressForm.findField('salutation').getValue();
	var eoriNumber = addressForm.findField('eoriNumber').getValue();
	var company = addressForm.findField('company').getValue();
	var department = addressForm.findField('department').getValue();
        formComponent.setLoading(true);
	var errorMessage = '';
        var error = false;
	if ( !street || !zip || !city ||  !addressType || !subShopId || !addressName) {
	    errorMessage = 'Bitte Ihre Adressdaten/-auswahl nochmals prüfen. Es fehlen erforderliche Werte.';
	    error = true;
        }
		
          if (salutation == 'company' && !company) {
                  errorMessage = "Bitte Ihre Addressdaten nochmals prüfen. Es fehlt Firma.";
                  error = true;
            } 

	   if(salutation != 'company' && !lastName) { 
                  error = true;
                  errorMessage = "Bitte Ihre Addressdaten nochmals prüfen. Es fehlt Nachname.";
            }

	 if(error) {
            Ext.Msg.alert('Bitte Eingabe prüfen', errorMessage);
            formComponent.setLoading(false);
            addressForm.isValid();
            return;
           }


        Ext.Ajax.request({
            url: '{url controller=PostPluginConfiguration action="saveAddress"}',
            method: 'POST',
            params: {
                street: street,
                firstName: firstName,
                city: city,
                zip: zip,
                phone: phone,
                lastName: lastName,
                addressType: addressType,
                subShopId: subShopId,
                bankId: bankId,
                id: id,
		salutation: salutation,
                addressName: addressName,
                eoriNumber: eoriNumber,
                company: company,
                department: department,
            },
            success: function (response) {
                formComponent.setLoading(false);
                response = Ext.JSON.decode(response.responseText);
                if (response.success) {
                    Shopware.Notification.createGrowlMessage('Speichern Erfolgreich');
                    Ext.Msg.alert('GESPEICHERT', 'Adresse Speichern Erfolgreich');
                    // reload data after successful saving
                    me.subApplication.getStore('Address').load();
                } else {
                    Shopware.Notification.createGrowlMessage('Es ist beim Speichern der Adresse ein Problem aufgetreten.', response.message);
                    Ext.Msg.alert('Es ist beim Speichern der Adresse ein Problem aufgetreten.', response.data.error.errorMessage + '\n' + response.data.error.errorMessageExtended);
                }
            }
        });
    }
});
//{/block}
