//{block name="backend/post_plugin_configuration/controller/contract"}
Ext.define('Shopware.apps.PostPluginConfiguration.controller.Bank', {
    extend: 'Enlight.app.Controller',
    refs: [
        { ref: 'window', selector: 'post_plugin_configuration-main-window' },
        { ref: 'bankForm', selector: 'post_plugin_bank-panel' }
    ],

    init: function() {
        var me = this;
        me.control({
            'post_plugin_configuration-bank-list': {
                deleteBank: me.onDeleteBank,
                onLoadBank: me.onLoadBank
            },
            'post_plugin_bank-panel-window': {
                addBank: me.onAddBank
            }
        });
        me.callParent(arguments);
    },
    onLoadBank: function (record) {
        var me = this;

        var formComponent = me.getBankForm();
        var bankForm = formComponent.getForm();
        bankForm.loadRecord(record);
    },
    onDeleteBank: function (record) {

        var me = this,
            store = me.subApplication.getStore('Bank'),
            message = 'Wollen Sie die Bank wirklich löschen?',
            title = 'Bank löschen';

        // we do not just delete - we are polite and ask the user if he is sure.
        Ext.MessageBox.confirm(title, message, function (response) {
            if (response !== 'yes') {
                return;
            }
            record.destroy({
                callback: function (data, operation) {
                    var records = operation.getRecords(),
                        record = records[0],
                        rawData = record.getProxy().getReader().rawData;

                    if (operation.success === true) {
                        //Shopware.Notification.createGrowlMessage(me.snippets.deleteOrder.successTitle, me.snippets.deleteOrder.successMessage, me.snippets.growlMessage);
                    } else {
                        //Shopware.Notification.createGrowlMessage(me.snippets.deleteOrder.failureTitle, me.snippets.deleteOrder.failureMessage + ' ' + rawData.message, me.snippets.growlMessage);
                    }
                    store.load();
                }
            });
        });
    },

    onAddBank: function (record) {
        var me = this;

        var formComponent = me.getBankForm();
        var bankForm = formComponent.getForm();
        var id = false;
        if(record) {
            id = record.get('id');
        }
        var bankAccountOwner = bankForm.findField('bankAccountOwner').getValue();
        var bic = bankForm.findField('bic').getValue();
        var iban = bankForm.findField('iban').getValue();
        var displayName = bankForm.findField('displayName').getValue();

        formComponent.setLoading(true);

        if (!bankAccountOwner || !bic || !iban) {
           Ext.Msg.alert('Bitte Eingabe prüfen', 'Bitte Ihre Auswahl nochmals prüfen. Es fehlen erforderliche Werte.');
            formComponent.setLoading(false)
	    bankForm.isValid();
            return;
        }

        Ext.Ajax.request({
            url: '{url controller=PostPluginConfiguration action="savePluginBank"}',
            method: 'POST',
            params: {
                bankAccountOwner: bankAccountOwner,
                displayName: displayName,
                bic: bic,
                iban: iban,
                id: id,
            },
            success: function (response) {
                formComponent.setLoading(false);
                response = Ext.JSON.decode(response.responseText);
                if (response.success) {
                    Shopware.Notification.createGrowlMessage('Speichern Erfolgreich');
                    Ext.Msg.alert('GESPEICHERT', 'Bankdaten Speichern Erfolgreich');
                    // reload data after successful saving
                    me.subApplication.getStore('Bank').load();
                } else {
                    Shopware.Notification.createGrowlMessage('Es ist beim Speichern der Bank ein Problem aufgetreten.', response.message);
                    Ext.Msg.alert('Es ist beim Speichern der Bank ein Problem aufgetreten.', response.data.error.errorMessage + '\n' + response.data.error.errorMessageExtended);
                }
            }
        });
    }
});
//{/block}
