//{block name="backend/post_plugin_configuration/view/detail/address_form"}
Ext.define('Shopware.apps.PostPluginConfiguration.view.detail.AddressForm', {
    extend: 'Ext.form.Panel',
    layout: 'fit',
    cls: 'shopware-form',
    alias: 'widget.post_plugin_address-panel',
    bodyPadding: 10,
    border: false,
    defaults: {
        anchor: '100%'
    },

    initComponent: function () {

        var me = this;
        if (me.record) {
            me.store = me.addressStore;
            me.store.load();

            me.store.load({
                callback: function () {
                    me.getForm().findField('subShopId').select(me.record.get('activeShopId'));
                    me.getForm().findField('bankId').select(me.record.get('bank'));
                }
            });
            me.store.load({
                callback: function () {
                    me.loadRecord(me.record);

                }
            });
        }
	
        Ext.tip.QuickTipManager.init();

        me.items = me.createContainer();
        me.callParent(arguments);
    },

    createContainer: function () {

        var container, me = this;
        container = Ext.create('Ext.container.Container', {
            layout: 'fit',
            border: false,
            items: [me.createAddressFormElements()]
        });

        return container;
    },

    getRowDefaults: function () {
        var me = this;
        return {
            columnWidth: 0.5,
            labelWidth: 155,
            labelStyle: 'font-weight: 700;',
            style: {
                margin: '0 15px 10px 15px'
            },
            store: me.store,
            xtype: 'textfield',
            allowBlank: false
        }
    },


    createAddressFormElements: function () {
        var row1, row2, row3, row4, row5, row6, row7, row8, me = this;

        row1 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow1Elements()
        });

        row2 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow2Elemenets()
        });

        row3 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow3Elements()
        });

        row4 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow4Elements()
        });

        row5 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow5Elements()
        });

        row6 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow6Elements()
        });

        row7 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow7Elements()
        });

	  row8 = Ext.create('Ext.container.Container', {
            columnWidth: 1,
            border: false,
            layout: 'column',
            defaults: me.getRowDefaults(),
            items: me.createRow8Elements()
        });

        return Ext.create('Ext.form.FieldSet', {
            layout: 'column',
            bodyPadding: 5,
            columnWidth: 1,
            stateful: true,
            title: 'Absenderdaten',
            items: [row1, row2, row8, row3, row4, row5, row6, row7]
        });
    },

    createRow1Elements: function () {
        var me = this;
        return [
            {
                name: 'addressName',
                fieldLabel: 'Adressname',
                value: 'Adresse ' + (me.addressStore.getCount() + 1),
            },
        ];
    },
    createRow2Elemenets: function () {
        return [
            {
                name: 'salutation',
                fieldLabel: 'Anrede',
                xtype: 'combobox',
                displayField: 'name',
                valueField: 'value',
                store: Ext.create('Ext.data.Store', {
                    fields: ['name', 'value'],
                    data: [
                        { name: 'Firma', value: 'company'},
                        { name: 'Frau', value: 'mrs'},
                        { name: 'Herr', value: 'mr'}
                    ]
                }
		)
            },
		 {
                name:'eoriNumber',
                fieldLabel: 'EORI-Nummer',
            	allowBlank: true
            },
        ];
    },

    createRow3Elements: function () {
        return [
            {
                name: 'firstName',
                fieldLabel: 'Vorname',
            	allowBlank: true
            },
            {
                name: 'lastName',
                fieldLabel: 'Nachname',
            	allowBlank: true
            },
        ];
    },

    createRow4Elements: function () {
        return [
            {
                name: 'street',
                fieldLabel: 'Straße/Hausnummer'
            },
            {
                name: 'city',
                fieldLabel: 'Ort'
            },


        ];
    },

    createRow5Elements: function () {
        return [
            {
                name: 'zip',
                fieldLabel: 'Postleitzahl'
            },
            {
                name: 'country',
                fieldLabel: 'Land',
                readOnly: true,
                value: 'AT'
            },

        ];
    },

    createRow6Elements: function () {
         var me = this;
	
	return [
                        {
                name: 'phone',
                fieldLabel: 'Telefonnummer',
            	allowBlank: true,
		helpText: "Pflichtangabe für Postexpress und Sendung in Drittländer",
            },
            {
                name: 'addressType',
                fieldLabel: 'Adresse',
                xtype: 'combobox',
                queryMode: 'local',
                displayField: 'name',
                valueField: 'value',
                store: Ext.create('Ext.data.Store', {
                    fields: ['name', 'value'],
                    data: [
                        { name: 'Versand- & Rücksendungsadresse', value: 'both'},
                        { name: 'Versandadresse', value: 'sender'},
                        { name: 'Rücksendungsadresse', value: 'return'},
                    ]
                } )
            },
        ];
    },

    createRow7Elements: function () {
        var me = this;
	return [
            {
                name: 'bankId',
                fieldLabel: 'Bank',
                xtype: 'combobox',
                displayField: 'iban',
                valueField: 'id',
            	allowBlank: true,
                store: me.bankStore,
            },
            {
                name: 'subShopId',
                fieldLabel: 'ShopId',
                xtype: 'combobox',
                displayField: 'name',
                valueField: 'id',
                store: me.subShopStore,
            },


        ];
    },
    createRow8Elements: function () {
        return [
            {
                name: 'company',
                fieldLabel: 'Firma'
            },
            {
                name: 'department',
                fieldLabel: 'Abteilung'
            },
        ];
    },
});
//{/block}

