//{block name="backend/post_plugin_configuration/view/list/post_plugin_configuration_address"}
Ext.define('Shopware.apps.PostPluginConfiguration.view.list.Address', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.post_plugin_configuration-address-list',
    cls: Ext.baseCSSPrefix + 'document-grid',
    autoScroll: true,
    overflow: 'scroll',
    stateful: true,
    minHeight: 50,
    maxHeight: 400,

    snippets: {
        columns: {
            addressName: "Name",
            firstName: "Vorname",
            salutation: "Anrede",
            lastName: "Nachname",
            phone: "Telefon",
            street: 'Strasse',
            city: 'Ort',
            zip: 'Postleitzahl',
            country: 'Land',
            addresstype: 'Typ',
            shop: 'Shop',
            bank: 'Bank',
	    eoriNumber: 'EORI-Nummer',
	    company: 'Firma',
	    department: 'Abteilung',
        }
    },
    initComponent: function() {
        var me = this;
        me.store = me.addressStore;
        me.store.load();
	me.subShopStore.load();
        me.dockedItems = [me.getToolbar()];
        me.columns = [
           
		 { header: me.snippets.columns.addressName, dataIndex: 'addressName', flex: 1,  },
            { header: me.snippets.columns.firstName, dataIndex: 'firstName', flex: 1,  },
            { header: me.snippets.columns.lastName, dataIndex: 'lastName', flex: 1,  },
            { header: me.snippets.columns.salutation, dataIndex: 'salutation', flex: 1, scope: me, renderer: me.salutationColumn  },
            { header: me.snippets.columns.phone, dataIndex: 'phone', flex: 1,  },
            { header: me.snippets.columns.street, dataIndex: 'street', flex: 1,  },
            { header: me.snippets.columns.city, dataIndex: 'city', flex: 1,  },
             { header: me.snippets.columns.eoriNumber, dataIndex: 'eoriNumber', flex: 1,  },
           { header: me.snippets.columns.zip, dataIndex: 'zip', flex: 1,  },
            { header: me.snippets.columns.addresstype, dataIndex: 'addressType', flex: 1, renderer: me.labelTypeColumn  },
	    { header: me.snippets.columns.shop, dataIndex: 'activeShopId', renderer: me.shopColumn, flex: 1,  },
	    { header: me.snippets.columns.company, dataIndex: 'company',  flex: 1,  },
	    { header: me.snippets.columns.department, dataIndex: 'department',  flex: 1,  },
    
	me.createActionColumn()
        ];


        me.callParent(arguments);
    },
   /**
 *      * Renderer function
 *           * @param value
 *                * @param metaData
 *                     * @param record
 *                          */
    salutationColumn: function (value, metaData, record) {
        var me = this;
        switch (value) {
            case 'mr':
                return 'Herr';
            case 'mrs':
                return 'Frau';
	    case 'company':
		return 'Firma';
            default:
                return value;
        }
    },

    /**
 *      * Renderer function of the user column
 *           * @param value
 *                * @param metaData
 *                     * @param record
 *                          */
    bankColumn: function(value, metaData, record) {
	var me = this;
        if (me.bankStore) {
            var bank = me.bankStore.getById(value);
		if(bank) {

            return bank.get('iban');
	}
	return value;
        } else {
            return value;
        }
    },

	    /**
 *      * Renderer function
 *           * @param value
 *                * @param metaData
 *                     * @param record
 *                          */
    shopColumn: function (value, metaData, record) {
        var me = this;
        if (me.subShopStore) {
            var bank = me.subShopStore.getById(value);
            if (bank) {
                return bank.get('title');
            }
            return value;
        } else {
            return value;
        }
    },
	   /**
 *      * Renderer function
 *           * @param value
 *                * @param metaData
 *                     * @param record
 *                          */
    labelTypeColumn: function (value, metaData, record) {
        var me = this;
        switch (value) {
            case 'both':
                return 'Versand- & Rücksendungsadresse';
            case 'sender':
                return 'Versandadresse';
            case 'return':
                return 'Rücksendungsadresse';
            default:
                return value;
        }
    },

    getToolbar: function() {
        var me = this,
            dockedItems = [];

        var addAddressButton = Ext.create('Ext.button.Button', {
            iconCls: 'sprite-plus-circle',
            text: 'hinzufügen',
            scope: me,
            handler:function () {
                me.showAddressDetails();
            }
        });

        return Ext.create('Ext.toolbar.Toolbar', {
            dock:'top',
            ui: 'shopware-ui',
            items:[
                addAddressButton,
                '->',]
        });
    },

    showAddressDetails: function (record) {
        var me = this;
        Ext.create('Shopware.apps.PostPluginConfiguration.view.detail.AddressWindow', {
            addressStore: me.addressStore,
            subShopStore: me.subShopStore,
            bankStore: me.bankStore,
            record: record
        }).show();
    },

    createActionColumn: function() {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 60,
            items: [
                me.createAdressLoadColumn(),
                me.createAdressDeleteColumn()
            ]
        });
    },

    createAdressLoadColumn: function() {
        var me = this;
        return {
            iconCls:'sprite-pencil',
            tooltip: 'Addresse bearbeiten',

            handler:function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.showAddressDetails(record);
                //me.fireEvent('loadAddress', record);
            }
        };
    },

    createAdressDeleteColumn: function() {
        var me = this;
        return {
            iconCls:'sprite-minus-circle-frame',
            tooltip: 'Addresse löschen?',

            handler:function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.fireEvent('deleteAddress', record);
            }
        };
    },

    registerEvents: function() {
        this.addEvents(
            'deleteAddress'
        );
    }
});
//{/block}

