//{block name="backend/post_plugin_configuration/view/list/post_plugin_configuration_bank"}
Ext.define('Shopware.apps.PostPluginConfiguration.view.list.Bank', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.post_plugin_configuration-bank-list',
    cls: Ext.baseCSSPrefix + 'document-grid',
    autoScroll: true,
    overflow: 'scroll',
    stateful: true,
    minHeight: 50,
    maxHeight: 250,
    snippets: {
        columns: {
            bankAccountOwner: 'Kontoinhaber',
            displayName: 'Anzeigename',
            bic: 'BIC',
            iban: 'IBAN',
        }
    },
    initComponent: function () {
        var me = this;
        me.store = me.bankStore;
        me.store.load();
        me.dockedItems = [me.getToolbar()];
        me.columns = [
            { header: 'id', dataIndex: 'id', flex: 1,},
            { header: me.snippets.columns.displayName, dataIndex: 'displayName', flex: 1,},
            { header: me.snippets.columns.bankAccountOwner, dataIndex: 'bankAccountOwner', flex: 1,},
            { header: me.snippets.columns.bic, dataIndex: 'bic', flex: 1,},
            { header: me.snippets.columns.iban, dataIndex: 'iban', flex: 1,},
            me.createActionColumn()
        ];


        me.callParent(arguments);
    },

    getToolbar: function () {
        var me = this,
            dockedItems = [];

        var addAddressButton = Ext.create('Ext.button.Button', {
            iconCls: 'sprite-plus-circle',
            text: 'hinzufügen',
            scope: me,
            handler: function () {
                me.showBankDetails();
            }
        });

        return Ext.create('Ext.toolbar.Toolbar', {
            dock: 'top',
            ui: 'shopware-ui',
            items: [
                addAddressButton,
                '->',]
        });
    },

    showBankDetails: function (record) {
        var me = this;
        Ext.create('Shopware.apps.PostPluginConfiguration.view.detail.BankWindow', {
            bankStore: me.bankStore,
            record: record
        }).show();
    },

    createBankLoadColumn: function () {
        var me = this;
        return {
            iconCls: 'sprite-pencil',
            tooltip: 'Bank bearbeiten',

            handler: function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.showBankDetails(record);
            }
        };
    },

    createActionColumn: function () {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 60,
            items: [
                me.createBankLoadColumn(),
                me.createBankDeleteColumn()
            ]
        });
    },

    createBankDeleteColumn: function () {
        var me = this;
        return {
            iconCls: 'sprite-minus-circle-frame',
            tooltip: 'Bankdaten löschen?',

            handler: function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.fireEvent('deleteBank', record);
            }
        };
    },

    registerEvents: function () {
        this.addEvents(
            'deleteBank'
        );
    }
});
//{/block}

