//{block name="backend/post_plugin_configuration/view/list/post_plugin_configuration_contract"}
Ext.define('Shopware.apps.PostPluginConfiguration.view.list.Contract', {
    extend: 'Ext.grid.Panel',
    alias: 'widget.post_plugin_configuration-contract-list',
    cls: Ext.baseCSSPrefix + 'document-grid',
    autoScroll: true,
    overflow: 'scroll',
    stateful: true,
    minHeight: 50,
    maxHeight: 400,

    snippets: {
        columns: {
            identifier: 'Kennung',
            orgUnitId: 'orgUnitID',
            orgUnitGUID: 'orgUnitGUID',
            license: 'ClientID'
        }
    },
    initComponent: function() {
        var me = this;
        me.store = me.contractStore;
        me.store.load();
        me.dockedItems = [me.getToolbar()];
        me.columns = [
            { header: me.snippets.columns.identifier, dataIndex: 'identifier', flex: 0.6,  },
            { header: me.snippets.columns.license, dataIndex: 'license', flex: 0.6,  },
            { header: me.snippets.columns.orgUnitId, dataIndex: 'contractnumber', flex: 0.6,  },
            { header: me.snippets.columns.orgUnitGUID, dataIndex: 'unitGUID', flex: 1,  },
            me.createActionColumn()
        ];

        me.callParent(arguments);
    },

    getToolbar: function () {
        var me = this;

        var addContractButton = Ext.create('Ext.button.Button', {
            iconCls: 'sprite-plus-circle',
            text: 'hinzufügen',
            scope: me,
            handler: function () {
                me.showContractDetails();
            }
        });

        return Ext.create('Ext.toolbar.Toolbar', {
            dock: 'top',
            ui: 'shopware-ui',
            items: [
                addContractButton,
                '->',]
        });
    },

    createContractLoadColumn: function () {
        var me = this;
        return {
            iconCls: 'sprite-pencil',
            tooltip: 'Bank bearbeiten',

            handler: function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.showContractDetails(record);
            }
        };
    },

    showContractDetails: function (record) {
        var me = this;
        Ext.create('Shopware.apps.PostPluginConfiguration.view.detail.ContractWindow', {
            contractStore: me.contractStore,
            record: record
        }).show();
    },

    createActionColumn: function() {
        var me = this;

        return Ext.create('Ext.grid.column.Action', {
            width: 60,
            items: [
                me.createContractDeleteColumn(),
                me.createContractLoadColumn()
            ]
        });
    },

    createContractDeleteColumn: function() {
        var me = this;
        return {
            iconCls:'sprite-minus-circle-frame',
            tooltip: 'Kennung löschen?',

            handler:function (view, rowIndex, colIndex, item) {
                var store = view.getStore(),
                    record = store.getAt(rowIndex);
                me.fireEvent('deleteContract', record);
            }
        };
    },

    registerEvents: function() {
        this.addEvents(
            'deleteContract'
        );
    }
});
//{/block}

