<?php
namespace Acl\GreenField\Modules\Plc\Shipment;

use JsonSerializable;
use Acl\GreenField\Exceptions\GreenFieldSDKException;

/**
 * GreenField Plc Module - Shipment Article
 *
 * Contains the article data for the shipping endpoint.
 *
 * @package   ACL\GreenFeild-PHP-SDK
 * @author    Marvin Tappauf <marvin.tappauf@acl.at>
 * @copyright 2020 (c) ACL GmbH
 * @license   Proprietary
 * @link      http://gitlab.acl.at/tools/greenfield-php-sdk
 * @version   1.0
 */
class ColloRowArticle implements JsonSerializable
{

    /**
     * Article parameters
     *
     * The parameters are set through the magic set method.
     *
     * Available parameters:
     * * articleName
     * * articleNumber
     * * consumerUnitNetWeight
     * * countryOfOriginId
     * * currencyID
     * * customsOptionID
     * * hsTariffNumber
     * * quantity
     * * unitID
     * * valueOfGoodsPerUnit
     *
     * @var array
     */
    protected $params = [
        "articleName"               =>  "",
        "articleNumber"             =>  "",
        "consumerUnitNetWeight"     =>  "",
        "countryOfOriginID"         =>  "",
        "currencyID"                =>  "",
        "customsOptionID"           =>  "",
        "hSTariffNumber"            =>  "",
        "quantity"                  =>  "",
        "unitID"                    =>  "",
        "valueOfGoodsPerUnit"       =>  "",
    ];

    /**
     * Parameter mapping
     *
     * Used for mapping when encoding to JSON.
     *
     * @var array
     */
    protected $mapping = [
        "articleName"               =>  "articleName",
        "articleNumber"             =>  "articleNumber",
        "consumerUnitNetWeight"     =>  "consumerUnitNetWeight",
        "countryOfOriginID"         =>  "countryOfOriginID",
        "currencyID"                =>  "currencyID",
        "customsOptionID"           =>  "customsOptionID",
        "hSTariffNumber"            =>  "hSTariffNumber",
        "quantity"                  =>  "quantity",
        "unitID"                    =>  "unitID",
        "valueOfGoodsPerUnit"       =>  "valueOfGoodsPerUnit",
    ];

    /**
     * Serialize address data for JSON encoding.
     *
     * @return array
     */
    public function jsonSerialize()
    {
        $data = [];
        foreach ($this->params as $k => $v) {
            $data[$this->mapping[$k]] = $v;
        }
        return $data;
    }

    /**
     * Magic setter to set the article data to the article object. If the parameter
     * does not exist in the $params array, an exception is thrown.
     *
     * @param string $name Paremeter name
     * @param string $value Parameter value
     * @return void
     *
     * @throws \Acl\GreenField\Exceptions\GreenFieldSDKException
     */
    public function __set($name, $value)
    {
        if (isset($this->params[$name]) === false) {
            throw new GreenFieldSDKException(
                "Parameter '{$name}' does not exist in the parameter list."
            );
        }

        $this->params[$name] = $value;
    }
}

