<?php
namespace Acl\GreenField\Modules\Plc\Shipment;

use JsonSerializable;
use Acl\GreenField\Exceptions\GreenFieldSDKException;

/**
 * GreenField Plc Module - Shipment DocumentEntry
 *
 * Contains the documentEntry data for the shipping endpoint.
 *
 * @package   ACL\GreenFeild-PHP-SDK
 * @author    Marvin Tappauf <marvin.tappauf@acl.at>
 * @copyright 2020 (c) ACL GmbH
 * @license   Proprietary
 * @link      http://gitlab.acl.at/tools/greenfield-php-sdk
 * @version   1.0
 */
class DocumentEntry implements JsonSerializable
{
    /**
     * Document parameters
     *
     * The parameters are set through the magic set method.
     *
     * Available parameters:
     * * documentId
     * * number
     * * quantity
     * @var array
     */
    protected $params = [
        "documentID"             =>  "",
        "number"                 =>  "",
        "quantity"               =>  "",
    ];

    /**
     * Parameter mapping
     *
     * Used for mapping when encoding to JSON.
     *
     * @var array
     */
    protected $mapping = [
        "documentID"             =>  "documentID",
        "number"                 =>  "number",
        "quantity"               =>  "quantity",
    ];

    /**
     * Serialize address data for JSON encoding.
     *
     * @return array
     */
    public function jsonSerialize()
    {
        $data = [];
        foreach ($this->params as $k => $v) {
            $data[$this->mapping[$k]] = $v;
        }
        return $data;
    }

    /**
     * Magic setter to set the article data to the article object. If the parameter
     * does not exist in the $params array, an exception is thrown.
     *
     * @param string $name Paremeter name
     * @param string $value Parameter value
     * @return void
     *
     * @throws \Acl\GreenField\Exceptions\GreenFieldSDKException
     */
    public function __set($name, $value)
    {
        if (isset($this->params[$name]) === false) {
            throw new GreenFieldSDKException(
                "Parameter '{$name}' does not exist in the parameter list."
            );
        }

        $this->params[$name] = $value;
    }
}

